@@ -1,27 +1,30 @@
 /*****************************************************************
-
-  audioconverter.c
- 
-  Copyright (c) 2003-2006 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
- 
-  http://gmerlin.sourceforge.net
- 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
- 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
- 
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
 #include <audio.h>
+#include <libsamplerate/common.h>
 #include <mix.h>
 #include <accel.h>
 
@@ -62,7 +65,7 @@ static void audio_converter_cleanup(gavl_audio_converter_t* cnv)
   while(ctx)
     {
     ctx = cnv->contexts->next;
-    if(ctx)
+    if(ctx && cnv->contexts->output_frame)
       gavl_audio_frame_destroy(cnv->contexts->output_frame);
     destroy_context(cnv->contexts);
     cnv->contexts = ctx;
@@ -85,7 +88,7 @@ gavl_audio_convert_context_create(gavl_audio_format_t  * input_format,
   gavl_audio_convert_context_t * ret;
   ret = calloc(1, sizeof(*ret));
   gavl_audio_format_copy(&(ret->input_format),  input_format);
-  gavl_audio_format_copy(&(ret->output_format), input_format);
+  gavl_audio_format_copy(&(ret->output_format), output_format);
 
   return ret;
   }
@@ -94,7 +97,6 @@ static void adjust_format(gavl_audio_format_t * f)
   {
   if(f->num_channels == 1)
     f->interleave_mode = GAVL_INTERLEAVE_NONE;
-
   
   if((f->num_channels == 2) &&
      (f->interleave_mode == GAVL_INTERLEAVE_2))
@@ -114,6 +116,7 @@ static void add_context(gavl_audio_converter_t* cnv,
     cnv->last_context = ctx;
     cnv->contexts     = ctx;
     }
+  ctx->output_format.samples_per_frame = 0;
   cnv->current_format = &(ctx->output_format);
   cnv->num_conversions++;
   }
@@ -130,15 +133,50 @@ static void dump_context(gavl_audio_convert_context_t * ctx)
   }
 #endif
 
-int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
-                              const gavl_audio_format_t * input_format,
-                              const gavl_audio_format_t * output_format)
+static void put_samplerate_context(gavl_audio_converter_t* cnv,
+                                   gavl_audio_format_t * tmp_format,
+                                   int out_samplerate)
+  {
+  gavl_audio_convert_context_t * ctx;
+
+  /* Sampleformat conversion with GAVL_INTERLEAVE_2 is not supported */
+  if(cnv->current_format->interleave_mode == GAVL_INTERLEAVE_2)
+    {
+    tmp_format->interleave_mode = GAVL_INTERLEAVE_NONE;
+    ctx = gavl_interleave_context_create(&(cnv->opt),
+                                         cnv->current_format,
+                                         tmp_format);
+    add_context(cnv, ctx);
+    }
+  
+  if(cnv->current_format->sample_format < GAVL_SAMPLE_FLOAT)
+    {
+    tmp_format->sample_format = GAVL_SAMPLE_FLOAT;
+    ctx = gavl_sampleformat_context_create(&(cnv->opt),
+                                           cnv->current_format,
+                                           tmp_format);
+    add_context(cnv, ctx);
+    }
+  
+  tmp_format->samplerate = out_samplerate;
+  ctx = gavl_samplerate_context_create(&(cnv->opt),
+                                       cnv->current_format,
+                                       tmp_format);
+  add_context(cnv, ctx);
+  }
+
+int gavl_audio_converter_reinit(gavl_audio_converter_t* cnv)
   {
   int do_mix, do_resample;
   int i;
   gavl_audio_convert_context_t * ctx;
 
+  gavl_audio_format_t * input_format, * output_format;
+  
   gavl_audio_format_t tmp_format;
+
+  input_format = &cnv->input_format;
+  output_format = &cnv->output_format;
   
 #if 0
   fprintf(stderr, "Initializing audio converter, quality: %d, Flags: 0x%08x\n",
@@ -149,19 +187,10 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
   audio_converter_cleanup(cnv);
 
   /* Copy formats and options */
-
-  gavl_audio_format_copy(&(cnv->input_format), input_format);
-  gavl_audio_format_copy(&(cnv->output_format), output_format);
-
-  adjust_format(&(cnv->input_format));
-  adjust_format(&(cnv->output_format));
-
-  if(cnv->input_format.samples_per_frame < cnv->output_format.samples_per_frame)
-    cnv->input_format.samples_per_frame = cnv->output_format.samples_per_frame;
-  else
-    cnv->output_format.samples_per_frame = cnv->input_format.samples_per_frame;
   
   memset(&tmp_format, 0, sizeof(tmp_format));
+  gavl_audio_format_copy(&tmp_format, &cnv->input_format);
+  
   cnv->current_format = &(cnv->input_format);
     
   /* Check if we must mix */
@@ -190,40 +219,17 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
       }
     }
 
-  /* Check is we must resample */
+  /* Check if we must resample */
 
   do_resample = (input_format->samplerate != output_format->samplerate) ? 1 : 0;
 
   /* Check for resampling. We take care, that we do resampling for the least possible channels */
 
-  if(do_resample && (!do_mix || (do_mix && (input_format->num_channels <= output_format->num_channels))))
-    {
-    /* Sampleformat conversion with GAVL_INTERLEAVE_2 is not supported */
-    if(cnv->current_format->interleave_mode == GAVL_INTERLEAVE_2)
-      {
-      tmp_format.interleave_mode = GAVL_INTERLEAVE_NONE;
-      ctx = gavl_interleave_context_create(&(cnv->opt),
-                                           cnv->current_format,
-                                           &tmp_format);
-      add_context(cnv, ctx);
-      }
-   
-    if(cnv->current_format->sample_format != GAVL_SAMPLE_FLOAT)
-      {
-      tmp_format.sample_format = GAVL_SAMPLE_FLOAT;
-      ctx = gavl_sampleformat_context_create(&(cnv->opt),
-                                             cnv->current_format,
-                                             &tmp_format);
-      add_context(cnv, ctx);
-      }
-
-    tmp_format.samplerate = output_format->samplerate;
-    ctx = gavl_samplerate_context_create(&(cnv->opt),
-                                         cnv->current_format,
-                                         &tmp_format);
-    add_context(cnv, ctx);
-    }
-
+  if(do_resample &&
+     (!do_mix ||
+      (do_mix && (input_format->num_channels <= output_format->num_channels))))
+    put_samplerate_context(cnv, &tmp_format, output_format->samplerate);
+  
   /* Check for mixing */
     
   if(do_mix)
@@ -237,7 +243,7 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
       add_context(cnv, ctx);
       }
     
-    if((cnv->current_format->sample_format != GAVL_SAMPLE_FLOAT) &&
+    if((cnv->current_format->sample_format < GAVL_SAMPLE_FLOAT) &&
        ((cnv->opt.quality > 3) ||
         (cnv->output_format.sample_format == GAVL_SAMPLE_FLOAT)))
       {
@@ -247,6 +253,17 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
                                              &tmp_format);
       add_context(cnv, ctx);
       }
+    else if((cnv->current_format->sample_format < GAVL_SAMPLE_DOUBLE) &&
+       ((cnv->opt.quality > 4) ||
+        (cnv->output_format.sample_format == GAVL_SAMPLE_DOUBLE)))
+      {
+      tmp_format.sample_format = GAVL_SAMPLE_DOUBLE;
+      ctx = gavl_sampleformat_context_create(&(cnv->opt),
+                                             cnv->current_format,
+                                             &tmp_format);
+      add_context(cnv, ctx);
+      }
+
     else if(gavl_bytes_per_sample(cnv->current_format->sample_format) <
             gavl_bytes_per_sample(cnv->output_format.sample_format))
       {
@@ -268,33 +285,11 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
 
   if(do_resample && do_mix && (input_format->num_channels > output_format->num_channels))
     {
-    /* Sampleformat conversion with GAVL_INTERLEAVE_2 is not supported */
-    if(cnv->current_format->interleave_mode == GAVL_INTERLEAVE_2)
-      {
-      tmp_format.interleave_mode = GAVL_INTERLEAVE_NONE;
-      ctx = gavl_interleave_context_create(&(cnv->opt),
-                                           cnv->current_format,
-                                           &tmp_format);
-      add_context(cnv, ctx);
-      }
-    
-    if(cnv->current_format->sample_format != GAVL_SAMPLE_FLOAT)
-      {
-      tmp_format.sample_format = GAVL_SAMPLE_FLOAT;
-      ctx = gavl_sampleformat_context_create(&(cnv->opt),
-                                             cnv->current_format,
-                                             &tmp_format);
-      add_context(cnv, ctx);
-      }
-    tmp_format.samplerate = output_format->samplerate;
-    ctx = gavl_samplerate_context_create(&(cnv->opt),
-                                         cnv->current_format,
-                                         &tmp_format);
-    add_context(cnv, ctx);
+    put_samplerate_context(cnv, &tmp_format, output_format->samplerate);
     }
-    
+  
   /* Check, if we must change the sample format */
-
+  
   if(cnv->current_format->sample_format != cnv->output_format.sample_format)
     {
     if(cnv->current_format->interleave_mode == GAVL_INTERLEAVE_2)
@@ -332,35 +327,86 @@ int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
   //  gavl_audio_format_dump(&(cnv->output_format));
   //  gavl_audio_format_dump(cnv->current_format);
 
-  /* Create temporary buffers */
-  ctx = cnv->contexts;
+  /* Set samples_per_frame of the first context
+     to zero to enable automatic allocation later on */
   
-  while(ctx)
+  cnv->input_format.samples_per_frame = 0;
+  return cnv->num_conversions;
+  }
+
+static void alloc_frames(gavl_audio_converter_t* cnv,
+                         int in_samples, double new_ratio)
+  {
+  gavl_audio_convert_context_t * ctx;
+  int out_samples_needed;  
+  if((cnv->input_format.samples_per_frame >= in_samples) && (new_ratio < 0.0))
+    return;
+ 
+  cnv->input_format.samples_per_frame = in_samples;
+  
+  /* Set the samples_per_frame member of all intermediate formats */
+  
+  ctx = cnv->contexts;
+  out_samples_needed = in_samples;  
+
+  while(ctx->next)
     {
-    //    dump_context(ctx);
-    if(ctx->next)
+    ctx->input_format.samples_per_frame = out_samples_needed;
+    
+    if(ctx->samplerate_converter)
       {
+      /* Varispeed */
+      if(new_ratio > 0.0)
+        {
+        out_samples_needed = 
+          (int)(0.5 * (ctx->samplerate_converter->ratio + new_ratio) * out_samples_needed) + 10;
+        }
+      /* Constant ratio */
+      else
+        {
+        out_samples_needed =
+          (out_samples_needed * ctx->output_format.samplerate) /
+          ctx->input_format.samplerate + 10;
+        }
+      }
+    if(ctx->output_format.samples_per_frame < out_samples_needed)
+      {
+      ctx->output_format.samples_per_frame = out_samples_needed + 1024;
+      if(ctx->output_frame)
+        gavl_audio_frame_destroy(ctx->output_frame);
       ctx->output_frame = gavl_audio_frame_create(&(ctx->output_format));
       ctx->next->input_frame = ctx->output_frame;
       }
     ctx = ctx->next;
     }
-  
-  return cnv->num_conversions;
+  }
+
+int gavl_audio_converter_init(gavl_audio_converter_t* cnv,
+                              const gavl_audio_format_t * input_format,
+                              const gavl_audio_format_t * output_format)
+  {
+  gavl_audio_format_copy(&(cnv->input_format), input_format);
+  gavl_audio_format_copy(&(cnv->output_format), output_format);
+
+  adjust_format(&(cnv->input_format));
+  adjust_format(&(cnv->output_format));
+  return gavl_audio_converter_reinit(cnv);
   }
 
 /* Convert audio */
 
 void gavl_audio_convert(gavl_audio_converter_t * cnv,
-                        gavl_audio_frame_t * input_frame,
+                        const gavl_audio_frame_t * input_frame,
                         gavl_audio_frame_t * output_frame)
   {
   int i;
   gavl_audio_convert_context_t * ctx;
-    
+  
   cnv->contexts->input_frame = input_frame;
   cnv->last_context->output_frame = output_frame;
-
+  
+  alloc_frames(cnv, input_frame->valid_samples, -1.0);
+  
   ctx = cnv->contexts;
   
   for(i = 0; i < cnv->num_conversions; i++)
@@ -371,6 +417,14 @@ void gavl_audio_convert(gavl_audio_converter_t * cnv,
       ctx->func(ctx);
       if(!ctx->output_frame->valid_samples)
         ctx->output_frame->valid_samples = ctx->input_frame->valid_samples;
+      
+      if(ctx->output_format.samplerate != ctx->input_format.samplerate)
+        ctx->output_frame->timestamp =
+          gavl_time_rescale(ctx->input_format.samplerate,
+                            ctx->output_format.samplerate,
+                            ctx->input_frame->timestamp);
+      else
+        ctx->output_frame->timestamp = ctx->input_frame->timestamp;
       }
     ctx = ctx->next;
     }
@@ -388,3 +442,127 @@ gavl_audio_converter_get_options(gavl_audio_converter_t * cnv)
   {
   return &(cnv->opt);
   }
+
+
+int gavl_audio_converter_init_resample(gavl_audio_converter_t * cnv,
+                                   const gavl_audio_format_t * format)
+{
+  gavl_audio_format_t tmp_format;
+  gavl_audio_convert_context_t * ctx;
+  
+  gavl_audio_format_copy(&(cnv->input_format), format);
+  gavl_audio_format_copy(&(cnv->output_format), format);
+  gavl_audio_format_copy(&(tmp_format), format);
+
+  adjust_format(&(cnv->input_format));
+  adjust_format(&(cnv->output_format));
+
+  // Delete previous conversions 
+  audio_converter_cleanup(cnv);
+
+  cnv->current_format = &(cnv->input_format);
+
+  put_samplerate_context(cnv, &tmp_format, cnv->output_format.samplerate);
+
+  /* put_samplerate will automatically convert sample format and interleave format 
+	* we need to check to see if it did or not and add contexts to convert back */
+  if(cnv->current_format->sample_format != cnv->output_format.sample_format)
+  {
+	  if(cnv->current_format->interleave_mode == GAVL_INTERLEAVE_2)
+	  {
+		  tmp_format.interleave_mode = GAVL_INTERLEAVE_NONE;
+		  ctx = gavl_interleave_context_create(&(cnv->opt),
+				  cnv->current_format,
+				  &tmp_format);
+		  add_context(cnv, ctx);
+	  }
+
+	  tmp_format.sample_format = cnv->output_format.sample_format;
+	  ctx = gavl_sampleformat_context_create(&(cnv->opt),
+			  cnv->current_format,
+			  &tmp_format);
+	  add_context(cnv, ctx);
+  }
+
+  /* Final interleaving */
+
+  if(cnv->current_format->interleave_mode != cnv->output_format.interleave_mode)
+  {
+	  tmp_format.interleave_mode = cnv->output_format.interleave_mode;
+	  ctx = gavl_interleave_context_create(&(cnv->opt),
+			  cnv->current_format,
+			  &tmp_format);
+	  add_context(cnv, ctx);
+  }
+
+  cnv->input_format.samples_per_frame = 0;
+
+  return cnv->num_conversions;
+}
+
+int gavl_audio_converter_set_resample_ratio(gavl_audio_converter_t * cnv, 
+		double ratio ) 
+{
+	int j;
+	gavl_audio_convert_context_t * ctx;
+
+	ctx = cnv->contexts;
+
+	if (ratio > SRC_MAX_RATIO  || ratio < 1/SRC_MAX_RATIO)
+		return 0;
+	
+	while(ctx) 
+	{
+		if (ctx->samplerate_converter != NULL)
+		{
+			for (j=0; j < ctx->samplerate_converter->num_resamplers; j++)
+				gavl_src_set_ratio( ctx->samplerate_converter->resamplers[j], ratio);
+		}
+		ctx->samplerate_converter->ratio = ratio;
+		ctx = ctx->next;
+	}
+	return 1;
+}
+
+void gavl_audio_converter_resample(gavl_audio_converter_t * cnv,
+		gavl_audio_frame_t * input_frame,
+		gavl_audio_frame_t * output_frame,
+		double ratio)
+{
+	gavl_audio_convert_context_t * ctx;
+
+	cnv->contexts->input_frame = input_frame;
+	cnv->last_context->output_frame = output_frame;
+
+	alloc_frames(cnv, input_frame->valid_samples, ratio);
+
+	ctx = cnv->contexts;
+
+	while(ctx) 
+	{
+		ctx->output_frame->valid_samples = 0;
+		if (ctx->samplerate_converter != NULL)
+		{
+			if (ctx->samplerate_converter->ratio != ratio )
+			{
+				//ctx->output_format.samplerate = ctx->input_format.samplerate * ratio;
+				ctx->samplerate_converter->ratio = ratio;
+				ctx->samplerate_converter->data.src_ratio = ratio;
+				//for (j=0; j < ctx->samplerate_converter->num_resamplers; j++)
+				//	gavl_src_set_ratio( ctx->samplerate_converter->resamplers[j], ratio);
+			}
+		}
+
+		if(ctx->func)
+		{
+			ctx->func(ctx);
+			// DO WE NEED THIS HERE???
+			if(!ctx->output_frame->valid_samples)
+				ctx->output_frame->valid_samples = ctx->input_frame->valid_samples;
+
+			ctx->output_frame->timestamp = ctx->input_frame->timestamp;
+
+		}
+		ctx = ctx->next;
+	}
+}
