@@ -1,21 +1,23 @@
 /*****************************************************************
- 
-  audioframe.c
- 
-  Copyright (c) 2003-2004 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
- 
-  http://gmerlin.sourceforge.net
- 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
- 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
- 
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 
 #include <string.h>
@@ -25,7 +27,7 @@
 #include <config.h>
 #include <gavl.h>
 #include <accel.h>
-
+#include <bswap.h>
 
 /* Taken from a52dec (thanks guys) */
 
@@ -56,6 +58,7 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
   switch(format->sample_format)
     {
     case GAVL_SAMPLE_U8:
+      ret->channel_stride = num_samples;
       ret->samples.u_8 =
         memalign(ALIGNMENT_BYTES, num_samples * format->num_channels);
 
@@ -64,6 +67,7 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
 
       break;
     case GAVL_SAMPLE_S8:
+      ret->channel_stride = num_samples;
       ret->samples.s_8 =
         memalign(ALIGNMENT_BYTES, num_samples * format->num_channels);
 
@@ -72,6 +76,7 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
 
       break;
     case GAVL_SAMPLE_U16:
+      ret->channel_stride = num_samples * 2;
       ret->samples.u_16 =
         memalign(ALIGNMENT_BYTES, 2 * num_samples * format->num_channels);
       for(i = 0; i < format->num_channels; i++)
@@ -79,6 +84,7 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
 
       break;
     case GAVL_SAMPLE_S16:
+      ret->channel_stride = num_samples * 2;
       ret->samples.s_16 =
         memalign(ALIGNMENT_BYTES, 2 * num_samples * format->num_channels);
       for(i = 0; i < format->num_channels; i++)
@@ -87,6 +93,7 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
       break;
 
     case GAVL_SAMPLE_S32:
+      ret->channel_stride = num_samples * 4;
       ret->samples.s_32 =
         memalign(ALIGNMENT_BYTES, 4 * num_samples * format->num_channels);
       for(i = 0; i < format->num_channels; i++)
@@ -95,12 +102,22 @@ gavl_audio_frame_create(const gavl_audio_format_t * format)
       break;
 
     case GAVL_SAMPLE_FLOAT:
+      ret->channel_stride = num_samples * sizeof(float);
       ret->samples.f =
         memalign(ALIGNMENT_BYTES, sizeof(float) * num_samples * format->num_channels);
 
       for(i = 0; i < format->num_channels; i++)
         ret->channels.f[i] = &(ret->samples.f[i*num_samples]);
 
+      break;
+    case GAVL_SAMPLE_DOUBLE:
+      ret->channel_stride = num_samples * sizeof(double);
+      ret->samples.d =
+        memalign(ALIGNMENT_BYTES, sizeof(double) * num_samples * format->num_channels);
+
+      for(i = 0; i < format->num_channels; i++)
+        ret->channels.d[i] = &(ret->samples.d[i*num_samples]);
+
       break;
     case GAVL_SAMPLE_NONE:
       {
@@ -118,12 +135,13 @@ void gavl_audio_frame_destroy(gavl_audio_frame_t * frame)
   free(frame);
   }
 
-void gavl_audio_frame_mute(gavl_audio_frame_t * frame,
-                           const gavl_audio_format_t * format)
+void gavl_audio_frame_mute_samples(gavl_audio_frame_t * frame,
+                                   const gavl_audio_format_t * format,
+                                   int num_samples)
   {
   int i;
   int imax;
-  imax = format->num_channels * format->samples_per_frame;
+  imax = format->num_channels * num_samples;
   
   switch(format->sample_format)
     {
@@ -149,23 +167,106 @@ void gavl_audio_frame_mute(gavl_audio_frame_t * frame,
       for(i = 0; i < imax; i++)
         frame->samples.s_32[i] = 0x00000000;
       break;
-      
     case GAVL_SAMPLE_FLOAT:
       for(i = 0; i < imax; i++)
         frame->samples.f[i] = 0.0;
       break;
+    case GAVL_SAMPLE_DOUBLE:
+      for(i = 0; i < imax; i++)
+        frame->samples.d[i] = 0.0;
+      break;
     }
-  frame->valid_samples = format->samples_per_frame;
+  frame->valid_samples = num_samples;
   }
 
-int gavl_audio_frame_copy(gavl_audio_format_t * format,
+void gavl_audio_frame_mute(gavl_audio_frame_t * frame,
+                           const gavl_audio_format_t * format)
+  {
+  gavl_audio_frame_mute_samples(frame,
+                                format,
+                                format->samples_per_frame);
+  }
+
+void gavl_audio_frame_mute_channel(gavl_audio_frame_t * frame,
+                                   const gavl_audio_format_t * format,
+                                   int channel)
+  {
+  int i;
+  int imax;
+  int offset = 0;
+  int advance = 0;
+
+  switch(format->interleave_mode)
+    {
+    case GAVL_INTERLEAVE_ALL:
+      offset = channel;
+      advance = format->num_channels;
+      break;
+    case GAVL_INTERLEAVE_NONE:
+      offset = channel * format->samples_per_frame;
+      advance = 1;
+      break;
+    case GAVL_INTERLEAVE_2:
+      if(channel & 1)
+        offset = (channel-1) * format->samples_per_frame + 1;
+      else
+        offset = channel * format->samples_per_frame;
+
+      if((channel == format->num_channels - 1) &&
+         (format->num_channels & 1))
+        advance = 1;
+      else
+        advance = 2;
+      
+      break;
+    }
+  
+  imax = format->samples_per_frame;
+  
+  switch(format->sample_format)
+    {
+    case GAVL_SAMPLE_NONE:
+      break;
+    case GAVL_SAMPLE_U8:
+      for(i = 0; i < imax; i++)
+        frame->samples.u_8[offset + advance * i] = 0x80;
+      break;
+    case GAVL_SAMPLE_S8:
+      for(i = 0; i < imax; i++)
+        frame->samples.u_8[offset + advance * i] = 0x00;
+      break;
+    case GAVL_SAMPLE_U16:
+      for(i = 0; i < imax; i++)
+        frame->samples.u_16[offset + advance * i] = 0x8000;
+      break;
+    case GAVL_SAMPLE_S16:
+      for(i = 0; i < imax; i++)
+        frame->samples.s_16[offset + advance * i] = 0x0000;
+      break;
+    case GAVL_SAMPLE_S32:
+      for(i = 0; i < imax; i++)
+        frame->samples.s_32[offset + advance * i] = 0x00000000;
+      break;
+    case GAVL_SAMPLE_FLOAT:
+      for(i = 0; i < imax; i++)
+        frame->samples.f[offset + advance * i] = 0.0;
+      break;
+    case GAVL_SAMPLE_DOUBLE:
+      for(i = 0; i < imax; i++)
+        frame->samples.d[offset + advance * i] = 0.0;
+      break;
+    }
+  }
+
+
+int gavl_audio_frame_copy(const gavl_audio_format_t * format,
                           gavl_audio_frame_t * dst,
-                          gavl_audio_frame_t * src,
+                          const gavl_audio_frame_t * src,
                           int out_pos,
                           int in_pos,
                           int out_size,
                           int in_size)
-      {
+  {
   int i;
   int bytes_per_sample;
   int samples_to_copy;
