@@ -1,24 +1,26 @@
 /*****************************************************************
-
-  audiooptions.c
-
-  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h> /* calloc, free */
-#include <string.h> /* calloc, free */
+#include <string.h> 
 
 #ifdef DEBUG
 #include <stdio.h>  
@@ -70,6 +72,22 @@ int gavl_audio_options_get_conversion_flags(gavl_audio_options_t * opt)
   return opt->conversion_flags;
   }
 
+int gavl_audio_options_get_quality(gavl_audio_options_t * opt)
+  {
+  return opt->quality;
+  }
+
+gavl_audio_dither_mode_t
+gavl_audio_options_get_dither_mode(gavl_audio_options_t * opt)
+  {
+  return opt->dither_mode;
+  }
+
+gavl_resample_mode_t
+gavl_audio_options_get_resample_mode(gavl_audio_options_t * opt)
+  {
+  return opt->resample_mode;
+  }
 
 void gavl_audio_options_set_defaults(gavl_audio_options_t * opt)
   {
