@@ -1,6 +1,26 @@
+/*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 static void RENAME(mix_1_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -20,7 +40,7 @@ static void RENAME(mix_1_to_1)(gavl_mix_output_channel_t * channel,
 
 
 static void RENAME(mix_2_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -42,7 +62,7 @@ static void RENAME(mix_2_to_1)(gavl_mix_output_channel_t * channel,
   }
 
 static void RENAME(mix_3_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -66,7 +86,7 @@ static void RENAME(mix_3_to_1)(gavl_mix_output_channel_t * channel,
   }
 
 static void RENAME(mix_4_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -93,7 +113,7 @@ static void RENAME(mix_4_to_1)(gavl_mix_output_channel_t * channel,
 
 
 static void RENAME(mix_5_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -121,7 +141,7 @@ static void RENAME(mix_5_to_1)(gavl_mix_output_channel_t * channel,
   }
 
 static void RENAME(mix_6_to_1)(gavl_mix_output_channel_t * channel,
-                               gavl_audio_frame_t * input_frame,
+                               const gavl_audio_frame_t * input_frame,
                                gavl_audio_frame_t * output_frame)
   {
   int i;
@@ -153,7 +173,7 @@ static void RENAME(mix_6_to_1)(gavl_mix_output_channel_t * channel,
   }
 
 static void RENAME(mix_all_to_1)(gavl_mix_output_channel_t * channel,
-                                 gavl_audio_frame_t * input_frame,
+                                 const gavl_audio_frame_t * input_frame,
                                  gavl_audio_frame_t * output_frame)
   {
   int i, j;
