@@ -1,3 +1,23 @@
+/*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 #include <audio.h>
 #include <mix.h>
 #include <accel.h>
@@ -101,6 +121,26 @@
 #define FACTOR(i)     channel->inputs[i].factor.f_float
 #define SAMPLE_TYPE   float
 #define TMP_TYPE      float
+#define ADJUST_TMP(i) CLAMP(i, -1.0f, 1.0f)
+
+#include "_mix_c.c"
+
+#undef RENAME
+#undef SRC
+#undef SETDST
+#undef FACTOR
+#undef SAMPLE_TYPE
+#undef TMP_TYPE
+#undef ADJUST_TMP
+
+/* Double */
+
+#define RENAME(a) a ## _double
+#define SRC(i,j)      input_frame->channels.d[SRC_INDEX(i)][j]
+#define SETDST(i,val) output_frame->channels.d[channel->index][i]=val
+#define FACTOR(i)     channel->inputs[i].factor.f_float
+#define SAMPLE_TYPE   double
+#define TMP_TYPE      double
 #define ADJUST_TMP(i) CLAMP(i, -1.0, 1.0)
 
 #include "_mix_c.c"
@@ -113,10 +153,11 @@
 #undef TMP_TYPE
 #undef ADJUST_TMP
 
+
 /* Copy routines */
 
 static void copy_8(gavl_mix_output_channel_t * channel,
-                   gavl_audio_frame_t * input_frame,
+                   const gavl_audio_frame_t * input_frame,
                    gavl_audio_frame_t * output_frame)
   {
   gavl_memcpy(output_frame->channels.s_8[channel->index],
@@ -125,7 +166,7 @@ static void copy_8(gavl_mix_output_channel_t * channel,
   }
 
 static void copy_16(gavl_mix_output_channel_t * channel,
-                    gavl_audio_frame_t * input_frame,
+                    const gavl_audio_frame_t * input_frame,
                     gavl_audio_frame_t * output_frame)
   {
   gavl_memcpy(output_frame->channels.s_16[channel->index],
@@ -134,7 +175,7 @@ static void copy_16(gavl_mix_output_channel_t * channel,
   }
 
 static void copy_32(gavl_mix_output_channel_t * channel,
-                    gavl_audio_frame_t * input_frame,
+                    const gavl_audio_frame_t * input_frame,
                     gavl_audio_frame_t * output_frame)
   {
   gavl_memcpy(output_frame->channels.s_32[channel->index],
@@ -142,6 +183,15 @@ static void copy_32(gavl_mix_output_channel_t * channel,
               input_frame->valid_samples*4);
   }
 
+static void copy_64(gavl_mix_output_channel_t * channel,
+                    const gavl_audio_frame_t * input_frame,
+                    gavl_audio_frame_t * output_frame)
+  {
+  gavl_memcpy(output_frame->channels.d[channel->index],
+              input_frame->channels.d[SRC_INDEX(0)],
+              input_frame->valid_samples*8);
+  }
+
 void gavl_setup_mix_funcs_c(gavl_mixer_table_t * t,
                             gavl_audio_format_t * f)
   {
@@ -159,6 +209,9 @@ void gavl_setup_mix_funcs_c(gavl_mixer_table_t * t,
     case 4:
       t->copy_func = copy_32;
       break;
+    case 8:
+      t->copy_func = copy_64;
+      break;
     }
   switch(f->sample_format)
     {
@@ -215,6 +268,15 @@ void gavl_setup_mix_funcs_c(gavl_mixer_table_t * t,
       t->mix_6_to_1 = mix_6_to_1_float;
       t->mix_all_to_1 = mix_all_to_1_float;
       break;
+    case GAVL_SAMPLE_DOUBLE:
+      t->mix_1_to_1 = mix_1_to_1_double;
+      t->mix_2_to_1 = mix_2_to_1_double;
+      t->mix_3_to_1 = mix_3_to_1_double;
+      t->mix_4_to_1 = mix_4_to_1_double;
+      t->mix_5_to_1 = mix_5_to_1_double;
+      t->mix_6_to_1 = mix_6_to_1_double;
+      t->mix_all_to_1 = mix_all_to_1_double;
+      break;
     case GAVL_SAMPLE_NONE:
       break;
     }
