@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  cputest.c
-
-  Copyright (c) 2001-2002 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <gavl.h>
 #include <config.h>
@@ -37,6 +39,7 @@
 #define MM_SSE      GAVL_ACCEL_SSE
 #define MM_SSE2     GAVL_ACCEL_SSE2
 #define MM_SSE3     GAVL_ACCEL_SSE3
+#define MM_SSSE3    GAVL_ACCEL_SSSE3
 #define MM_3DNOW    GAVL_ACCEL_3DNOW
 #define MM_3DNOWEXT GAVL_ACCEL_3DNOWEXT
 
@@ -104,6 +107,9 @@ int gavl_accel_supported()
             rval |= MM_SSE2;
         if (ecx & 1)
             rval |= MM_SSE3;
+        if (ecx & 0x00000200 )
+          rval |= MM_SSSE3;
+
     }
 
     cpuid(0x80000000, max_ext_level, ebx, ecx, edx);
