@@ -1,3 +1,24 @@
+/*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
+
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -35,6 +56,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
             case 4:
               return t->interleave_all_to_2_32;
               break;
+            case 8:
+              return t->interleave_all_to_2_64;
+              break;
             }
           break;
         case GAVL_INTERLEAVE_NONE:
@@ -51,6 +75,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
               case 4:
                 return t->interleave_all_to_none_stereo_32;
                 break;
+              case 8:
+                return t->interleave_all_to_none_stereo_64;
+                break;
               }
             }
           else
@@ -66,6 +93,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
               case 4:
                 return t->interleave_all_to_none_32;
                 break;
+              case 8:
+                return t->interleave_all_to_none_64;
+                break;
               }
             }
           break;
@@ -85,6 +115,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
             case 4:
               return t->interleave_2_to_all_32;
               break;
+            case 8:
+              return t->interleave_2_to_all_64;
+              break;
             }
           break;
         case GAVL_INTERLEAVE_2:
@@ -101,6 +134,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
             case 4:
               return t->interleave_2_to_none_32;
               break;
+            case 8:
+              return t->interleave_2_to_none_64;
+              break;
             }
           break;
         }
@@ -122,6 +158,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
               case 4:
                 return t->interleave_none_to_all_stereo_32;
                 break;
+              case 8:
+                return t->interleave_none_to_all_stereo_64;
+                break;
               }
             break;
             }
@@ -138,6 +177,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
               case 4:
                 return t->interleave_none_to_all_32;
                 break;
+              case 8:
+                return t->interleave_none_to_all_64;
+                break;
               }
             }
           break;
@@ -153,6 +195,9 @@ gavl_find_interleave_converter(gavl_interleave_table_t * t,
             case 4:
               return t->interleave_none_to_2_32;
               break;
+            case 8:
+              return t->interleave_none_to_2_64;
+              break;
             }
           break;
         case GAVL_INTERLEAVE_NONE:
