@@ -1,22 +1,23 @@
 /*****************************************************************
- 
-  mix.c
- 
-  Copyright (c) 2003-2004 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
- 
-  http://gmerlin.sourceforge.net
- 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
- 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
- 
-*****************************************************************/
-
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -60,6 +61,7 @@ static void output_channel_dump(gavl_mix_output_channel_t * c)
   int i;
   fprintf(stderr, "Output channel %d, num_inputs: %d, func: %p\n",
           c->index, c->num_inputs, c->func);
+  
   for(i = 0; i < c->num_inputs; i++)
     {
     input_channel_dump(&(c->inputs[i]));
@@ -70,7 +72,6 @@ static void output_channel_dump(gavl_mix_output_channel_t * c)
 void gavl_mix_audio(gavl_audio_convert_context_t * ctx)
   {
   int i;
-  //  fprintf(stderr, "MIX...");
   for(i = 0; i < ctx->output_format.num_channels; i++)
     {
     if(ctx->mix_matrix->output_channels[i].func)
@@ -78,18 +79,17 @@ void gavl_mix_audio(gavl_audio_convert_context_t * ctx)
                                                 ctx->input_frame,
                                                 ctx->output_frame);
     else
-      gavl_audio_frame_mute(ctx->output_frame, &(ctx->output_format));
-    //      fprintf(stderr, "Mixer function missing\n");
-    
+      /* This happens, if channels in the output are muted */
+      gavl_audio_frame_mute_channel(ctx->output_frame,
+                                    &(ctx->output_format), i);
     }
-  //  fprintf(stderr, "done\n");
   }
 
 #if 0
 
 static void dump_matrix(gavl_audio_format_t * in,
                         gavl_audio_format_t * out,
-                        float matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS])
+                        double matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS])
   {
   int i, j;
   fprintf(stderr, "Mix Matrix:\n");
@@ -106,11 +106,11 @@ static void dump_matrix(gavl_audio_format_t * in,
 
 #endif
 
-static void normalize_matrix(float ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
+static void normalize_matrix(double ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
                              int in_channels, int out_channels)
   {
-  float max_ampl;
-  float ampl;
+  double max_ampl;
+  double ampl;
   int i, j;
 
   /* Normalize matrix */
@@ -141,7 +141,7 @@ static void normalize_matrix(float ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
 #define IN_INDEX(id)  in_index  = gavl_channel_index(in, id);  if(in_index < 0)  goto fail;
 #define OUT_INDEX(id) out_index = gavl_channel_index(out, id); if(out_index < 0) goto fail;
 
-static void init_matrix(float ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
+static void init_matrix(double ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
                         gavl_audio_options_t * opt,
                         gavl_audio_format_t * in,
                         gavl_audio_format_t * out)
@@ -686,7 +686,7 @@ static void init_matrix(float ret[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
   }
 
 static void set_factor(gavl_mix_input_channel_t * ret,
-                       float fac, gavl_sample_format_t format)
+                       double fac, gavl_sample_format_t format)
   {
   switch(format)
     {
@@ -702,6 +702,7 @@ static void set_factor(gavl_mix_input_channel_t * ret,
       ret->factor.f_32 =    (int32_t)(fac * (INT32_MAX>>1) + 0.5);
       break;
     case GAVL_SAMPLE_FLOAT:
+    case GAVL_SAMPLE_DOUBLE:
       ret->factor.f_float = fac;
       break;
     case GAVL_SAMPLE_NONE:
@@ -710,7 +711,7 @@ static void set_factor(gavl_mix_input_channel_t * ret,
   }
 
 static void init_context(gavl_mix_matrix_t * ctx,
-                         float matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
+                         double matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS],
                          gavl_audio_format_t * in_format,
                          gavl_audio_format_t * out_format)
   {
@@ -728,10 +729,11 @@ static void init_context(gavl_mix_matrix_t * ctx,
     num_inputs = 0;
     for(j = 0; j < in_format->num_channels; j++)
       {
+      //      fprintf(stderr, "matrix[%d][%d]: %f\n",
+      //              i, j, matrix[i][j]);
+      
       if(matrix[i][j] != 0.0)
         {
-        //        fprintf(stderr, "Set factor %f %d %d\n",
-        //                matrix[i][j], i, j);
         set_factor(&(ctx->output_channels[i].inputs[num_inputs]),
                    matrix[i][j], in_format->sample_format);
         ctx->output_channels[i].inputs[num_inputs].index = j;
@@ -792,7 +794,7 @@ gavl_create_mix_matrix(gavl_audio_options_t * opt,
                         gavl_audio_format_t * in,
                         gavl_audio_format_t * out)
   {
-  float mix_matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS];
+  double mix_matrix[GAVL_MAX_CHANNELS][GAVL_MAX_CHANNELS];
   gavl_mix_matrix_t * ret;
   ret = calloc(1, sizeof(*ret));
 
