@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  sampleformat.c
-
-  Copyright (c) 2001-2002 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -39,7 +41,7 @@ void gavl_audio_dither_context_destroy(gavl_audio_dither_context_t* ctx)
   free(ctx);
   }
 
-static void convert_gdither_ni(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_ni_f(gavl_audio_convert_context_t * ctx)
   {
   int i;
   //  fprintf(stderr, "Convert gdither\n");
@@ -51,7 +53,7 @@ static void convert_gdither_ni(gavl_audio_convert_context_t * ctx)
     }
   }
 
-static void convert_gdither_s8_ni(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_s8_ni_f(gavl_audio_convert_context_t * ctx)
   {
   int i, j;
   for(i = 0; i < ctx->input_format.num_channels; i++)
@@ -65,7 +67,7 @@ static void convert_gdither_s8_ni(gavl_audio_convert_context_t * ctx)
     }
   }
 
-static void convert_gdither_u16_ni(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_u16_ni_f(gavl_audio_convert_context_t * ctx)
   {
   int i, j;
   for(i = 0; i < ctx->input_format.num_channels; i++)
@@ -79,7 +81,7 @@ static void convert_gdither_u16_ni(gavl_audio_convert_context_t * ctx)
     }
   }
 
-static void convert_gdither_i(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_i_f(gavl_audio_convert_context_t * ctx)
   {
   gdither_runf(ctx->dither_context->dither, 0,
                ctx->input_format.num_channels * ctx->input_frame->valid_samples,
@@ -87,7 +89,7 @@ static void convert_gdither_i(gavl_audio_convert_context_t * ctx)
                ctx->output_frame->samples.u_8);
   }
 
-static void convert_gdither_s8_i(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_s8_i_f(gavl_audio_convert_context_t * ctx)
   {
   int j, jmax;
   jmax = ctx->input_format.num_channels * ctx->input_frame->valid_samples;
@@ -99,7 +101,7 @@ static void convert_gdither_s8_i(gavl_audio_convert_context_t * ctx)
     ctx->output_frame->samples.s_8[j] ^= 0x80;
   }
 
-static void convert_gdither_u16_i(gavl_audio_convert_context_t * ctx)
+static void convert_gdither_u16_i_f(gavl_audio_convert_context_t * ctx)
   {
   int j, jmax;
   jmax = ctx->input_format.num_channels * ctx->input_frame->valid_samples;
@@ -113,6 +115,81 @@ static void convert_gdither_u16_i(gavl_audio_convert_context_t * ctx)
     ctx->output_frame->samples.u_16[j] ^= 0x8000;
   }
 
+/* */
+
+static void convert_gdither_ni_d(gavl_audio_convert_context_t * ctx)
+  {
+  int i;
+  //  fprintf(stderr, "Convert gdither\n");
+  for(i = 0; i < ctx->input_format.num_channels; i++)
+    {
+    gdither_run(ctx->dither_context->dither, 0, ctx->input_frame->valid_samples,
+                ctx->input_frame->channels.d[i],
+                ctx->output_frame->channels.u_8[i]);
+    }
+  }
+
+static void convert_gdither_s8_ni_d(gavl_audio_convert_context_t * ctx)
+  {
+  int i, j;
+  for(i = 0; i < ctx->input_format.num_channels; i++)
+    {
+    gdither_run(ctx->dither_context->dither, 0, ctx->input_frame->valid_samples,
+                 ctx->input_frame->channels.d[i],
+                 ctx->output_frame->channels.s_8[i]);
+
+    for(j = 0; j < ctx->input_frame->valid_samples; j++)
+      ctx->output_frame->channels.s_8[i][j] ^= 0x80;
+    }
+  }
+
+static void convert_gdither_u16_ni_d(gavl_audio_convert_context_t * ctx)
+  {
+  int i, j;
+  for(i = 0; i < ctx->input_format.num_channels; i++)
+    {
+    gdither_run(ctx->dither_context->dither, 0, ctx->input_frame->valid_samples,
+                 ctx->input_frame->channels.d[i],
+                 ctx->output_frame->channels.u_16[i]);
+
+    for(j = 0; j < ctx->input_frame->valid_samples; j++)
+      ctx->output_frame->channels.u_16[i][j] ^= 0x8000;
+    }
+  }
+
+static void convert_gdither_i_d(gavl_audio_convert_context_t * ctx)
+  {
+  gdither_run(ctx->dither_context->dither, 0,
+               ctx->input_format.num_channels * ctx->input_frame->valid_samples,
+               ctx->input_frame->samples.d,
+               ctx->output_frame->samples.u_8);
+  }
+
+static void convert_gdither_s8_i_d(gavl_audio_convert_context_t * ctx)
+  {
+  int j, jmax;
+  jmax = ctx->input_format.num_channels * ctx->input_frame->valid_samples;
+  gdither_run(ctx->dither_context->dither, 0,
+               jmax,
+               ctx->input_frame->samples.d,
+               ctx->output_frame->samples.s_8);
+  for(j = 0; j < jmax; j++)
+    ctx->output_frame->samples.s_8[j] ^= 0x80;
+  }
+
+static void convert_gdither_u16_i_d(gavl_audio_convert_context_t * ctx)
+  {
+  int j, jmax;
+  jmax = ctx->input_format.num_channels * ctx->input_frame->valid_samples;
+  
+  gdither_run(ctx->dither_context->dither, 0,
+              jmax,
+              ctx->input_frame->samples.d,
+              ctx->output_frame->samples.u_16);
+  
+  for(j = 0; j < jmax; j++)
+    ctx->output_frame->samples.u_16[j] ^= 0x8000;
+  }
 
 gavl_sampleformat_table_t * gavl_create_sampleformat_table(gavl_audio_options_t * opt,
                                                            gavl_interleave_mode_t interleave_mode)
@@ -190,7 +267,7 @@ gavl_sampleformat_context_create(gavl_audio_options_t * opt,
   
   if((dither_type == GAVL_AUDIO_DITHER_NONE) ||
      (gavl_bytes_per_sample(out_format->sample_format) > 2) ||
-     (in_format->sample_format != GAVL_SAMPLE_FLOAT))
+     (in_format->sample_format < GAVL_SAMPLE_FLOAT))
     {
     table = gavl_create_sampleformat_table(opt, in_format->interleave_mode);
     
@@ -210,38 +287,79 @@ gavl_sampleformat_context_create(gavl_audio_options_t * opt,
         dither_bit_depth = GDither8bit;
         dither_depth = 8;
 
-        if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
-          ret->func = convert_gdither_ni;
-        else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
-          ret->func = convert_gdither_i;
+        if(in_format->sample_format == GAVL_SAMPLE_DOUBLE)
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_ni_d;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_i_d;
+          }
+        else
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+          ret->func = convert_gdither_ni_f;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_i_f;
+          }
         
         break;
       case GAVL_SAMPLE_S8:
         dither_bit_depth = GDither8bit;
         dither_depth = 8;
 
-        if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
-          ret->func = convert_gdither_s8_ni;
-        else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
-          ret->func = convert_gdither_s8_i;
+        if(in_format->sample_format == GAVL_SAMPLE_DOUBLE)
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_s8_ni_d;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_s8_i_d;
+          }
+        else
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_s8_ni_f;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_s8_i_f;
+          }
         break;
       case GAVL_SAMPLE_U16:
         dither_bit_depth = GDither16bit;
         dither_depth = 16;
 
-        if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
-          ret->func = convert_gdither_u16_ni;
-        else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
-          ret->func = convert_gdither_u16_i;
+        if(in_format->sample_format == GAVL_SAMPLE_DOUBLE)
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_u16_ni_d;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_u16_i_d;
+          }
+        else
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_u16_ni_f;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_u16_i_f;
+          }
         break;
       case GAVL_SAMPLE_S16:
         dither_bit_depth = GDither16bit;
         dither_depth = 16;
 
-        if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
-          ret->func = convert_gdither_ni;
-        else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
-          ret->func = convert_gdither_i;
+        if(in_format->sample_format == GAVL_SAMPLE_DOUBLE)
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_ni_d;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_i_d;
+          }
+        else
+          {
+          if(in_format->interleave_mode == GAVL_INTERLEAVE_NONE)
+            ret->func = convert_gdither_ni_f;
+          else if(in_format->interleave_mode == GAVL_INTERLEAVE_ALL)
+            ret->func = convert_gdither_i_f;
+          
+          }
         break;
       default:
         fprintf(stderr, "BUG: Invalid dither initialization\n");
@@ -296,6 +414,9 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           return t->convert_u8_to_float;
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_u8_to_double;
+          break;
         case GAVL_SAMPLE_NONE:
           break;
         }
@@ -321,6 +442,9 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           return t->convert_s8_to_float;
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_s8_to_double;
+          break;
         case GAVL_SAMPLE_NONE:
           break;
         }
@@ -346,6 +470,10 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           return t->convert_u16_to_float;
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_u16_to_double;
+          break;
+          
         case GAVL_SAMPLE_NONE:
           break;
         }
@@ -371,6 +499,9 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           return t->convert_s16_to_float;
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_s16_to_double;
+          break;
         case GAVL_SAMPLE_NONE:
           break;
         }
@@ -396,6 +527,10 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           return t->convert_s32_to_float;
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_s32_to_double;
+          break;
+
         case GAVL_SAMPLE_NONE:
           break;
         }
@@ -421,6 +556,37 @@ gavl_audio_func_t gavl_find_sampleformat_converter(gavl_sampleformat_table_t * t
         case GAVL_SAMPLE_FLOAT:
           // Nothing
           break;
+        case GAVL_SAMPLE_DOUBLE:
+          return t->convert_float_to_double;
+          break;
+        case GAVL_SAMPLE_NONE:
+          break;
+        }
+      break;
+    case GAVL_SAMPLE_DOUBLE:
+      switch(out->sample_format)
+        {
+        case GAVL_SAMPLE_U8:
+          return t->convert_double_to_u8;
+          break;
+        case GAVL_SAMPLE_S8:
+          return t->convert_double_to_s8;
+          break;
+        case GAVL_SAMPLE_U16:
+          return t->convert_double_to_u16;
+          break;
+        case GAVL_SAMPLE_S16:
+          return t->convert_double_to_s16;
+          break;
+        case GAVL_SAMPLE_S32:
+          return t->convert_double_to_s32;
+          break;
+        case GAVL_SAMPLE_FLOAT:
+          return t->convert_double_to_float;
+          break;
+        case GAVL_SAMPLE_DOUBLE:
+          // Nothing
+          break;
         case GAVL_SAMPLE_NONE:
           break;
         }
