@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  video.c
-
-  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h> /* calloc, free */
 
@@ -23,9 +25,9 @@
 
 //#define DEBUG
 
-#ifdef DEBUG
+//#ifdef DEBUG
 #include <stdio.h>  
-#endif
+//#endif
 
 #include "gavl.h"
 #include "config.h"
@@ -111,14 +113,14 @@ static int add_context_csp(gavl_video_converter_t * cnv,
 
   if(!ctx->func)
     {
-#ifdef DEBUG
+#if 0
     fprintf(stderr, "Found no conversion from %s to %s\n",
             gavl_pixelformat_to_string(input_format->pixelformat),
             gavl_pixelformat_to_string(output_format->pixelformat));
 #endif
     return 0;
     }
-#ifdef DEBUG
+#if 0
   fprintf(stderr, "Doing pixelformat conversion from %s to %s\n",
           gavl_pixelformat_to_string(input_format->pixelformat),
           gavl_pixelformat_to_string(output_format->pixelformat));
@@ -134,9 +136,9 @@ static void scale_func(gavl_video_convert_context_t * ctx)
                           ctx->output_frame);
   }
 
-static void add_context_scale(gavl_video_converter_t * cnv,
-                       const gavl_video_format_t * input_format,
-                       const gavl_video_format_t * output_format)
+static int add_context_scale(gavl_video_converter_t * cnv,
+                             const gavl_video_format_t * input_format,
+                             const gavl_video_format_t * output_format)
   {
   gavl_video_options_t * scaler_options;
   
@@ -164,13 +166,15 @@ static void add_context_scale(gavl_video_converter_t * cnv,
   fprintf(stderr, "\n");
 #endif
   
-  gavl_video_scaler_init(ctx->scaler,
-                         input_format,
-                         output_format);
-  
+  if(!gavl_video_scaler_init(ctx->scaler,
+                             input_format,
+                             output_format))
+    {
+    //    fprintf(stderr, "Initializing scaler failed\n");
+    return 0;
+    }
   ctx->func = scale_func;
-  
-  
+  return 1;
   }
 
 static void deinterlace_func(gavl_video_convert_context_t * ctx)
@@ -192,17 +196,16 @@ static int add_context_deinterlace(gavl_video_converter_t * cnv,
   deinterlacer_options = gavl_video_deinterlacer_get_options(ctx->deinterlacer);
   gavl_video_options_copy(deinterlacer_options, &(cnv->options));
   
-  gavl_video_deinterlacer_init(ctx->deinterlacer,
-                               in_format);
+  if(!gavl_video_deinterlacer_init(ctx->deinterlacer,
+                                   in_format))
+    return 0;
   
   ctx->func = deinterlace_func;
   return 1;
   }
 
 
-int gavl_video_converter_init(gavl_video_converter_t * cnv,
-                              const gavl_video_format_t * input_format,
-                              const gavl_video_format_t * output_format)
+int gavl_video_converter_reinit(gavl_video_converter_t * cnv)
   {
   int csp_then_scale = 0;
   gavl_video_convert_context_t * tmp_ctx;
@@ -221,6 +224,12 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
   gavl_video_format_t tmp_format;
   gavl_video_format_t tmp_format1;
 
+  gavl_video_format_t * input_format;
+  gavl_video_format_t * output_format;
+
+  input_format = &cnv->input_format;
+  output_format = &cnv->output_format;
+  
   // #ifdef DEBUG
 #if 0
   //  fprintf(stderr, "Initializing video converter, quality: %d, Flags: 0x%08x\n",
@@ -261,9 +270,7 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
      (tmp_format.image_width  != output_format->image_width) ||
      (tmp_format.image_height != output_format->image_height) ||
      (tmp_format.pixel_width  != output_format->pixel_width) ||
-     (tmp_format.pixel_height != output_format->pixel_height) ||
-     ((cnv->options.quality > 3)
-      && (tmp_format.chroma_placement != output_format->chroma_placement)))
+     (tmp_format.pixel_height != output_format->pixel_height))
     {
     do_scale = 1;
     }
@@ -272,7 +279,9 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
      accurate conversion. This is especially true if the chroma subsampling
      ratios change or when the chroma placement becomes different */
     
-  if(((cnv->options.quality > 3) || do_scale))
+  if(((cnv->options.quality > 3) ||
+      (cnv->options.conversion_flags & GAVL_RESAMPLE_CHROMA) ||
+      do_scale))
     {
     if(do_csp)
       {
@@ -344,7 +353,7 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
       {
       if(!gavl_pixelformat_can_scale(input_format->pixelformat, tmp_csp))
         csp_then_scale = 1;
-#ifdef DEBUG
+#if 0
       fprintf(stderr, "converting %s -> %s -> %s (%d, %d)\n",
               gavl_pixelformat_to_string(input_format->pixelformat),
               gavl_pixelformat_to_string(tmp_csp),
@@ -356,7 +365,7 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
     
     if(csp_then_scale) /* csp then scale */
       {
-#ifdef DEBUG
+#if 0
       fprintf(stderr, "csp then scale\n");
 #endif
       /* csp (tmp_format -> tmp_format1) */
@@ -388,7 +397,8 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
       tmp_format1.chroma_placement = output_format->chroma_placement;
       tmp_format1.interlace_mode = output_format->interlace_mode;
       
-      add_context_scale(cnv, &tmp_format, &tmp_format1);
+      if(!add_context_scale(cnv, &tmp_format, &tmp_format1))
+        return -1;
 
       gavl_video_format_copy(&tmp_format, &tmp_format1);
       
@@ -396,7 +406,7 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
     /* scale then csp */
     else
       {
-#ifdef DEBUG
+#if 0
       fprintf(stderr, "scale then csp\n");
 #endif
       /* scale (tmp_format -> tmp_format1) */
@@ -419,7 +429,8 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
         }
       tmp_format1.chroma_placement = output_format->chroma_placement;
       
-      add_context_scale(cnv, &tmp_format, &tmp_format1);
+      if(!add_context_scale(cnv, &tmp_format, &tmp_format1))
+        return -1;
 
       gavl_video_format_copy(&tmp_format, &tmp_format1);
 
@@ -443,8 +454,9 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
 
   else if(do_scale)
     {
-    add_context_scale(cnv, &tmp_format,
-                      output_format);
+    if(!add_context_scale(cnv, &tmp_format,
+                          output_format))
+      return -1;
     }
 
   /* Now, create temporary frames for the contexts */
@@ -463,6 +475,16 @@ int gavl_video_converter_init(gavl_video_converter_t * cnv,
   return cnv->num_contexts;
   }
 
+int gavl_video_converter_init(gavl_video_converter_t * cnv,
+                              const gavl_video_format_t * input_format,
+                              const gavl_video_format_t * output_format)
+  {
+  gavl_video_format_copy(&cnv->input_format, input_format);
+  gavl_video_format_copy(&cnv->output_format, output_format);
+  return gavl_video_converter_reinit(cnv);
+  }
+
+
 gavl_video_options_t *
 gavl_video_converter_get_options(gavl_video_converter_t * cnv)
   {
@@ -474,7 +496,7 @@ gavl_video_converter_get_options(gavl_video_converter_t * cnv)
  ***************************************************/
 
 void gavl_video_convert(gavl_video_converter_t * cnv,
-                        gavl_video_frame_t * input_frame,
+                        const gavl_video_frame_t * input_frame,
                         gavl_video_frame_t * output_frame)
   {
   gavl_video_convert_context_t * tmp_ctx;
@@ -482,14 +504,18 @@ void gavl_video_convert(gavl_video_converter_t * cnv,
   cnv->first_context->input_frame = input_frame;
   cnv->last_context->output_frame = output_frame;
 
+  output_frame->timestamp = input_frame->timestamp;
+  output_frame->duration = input_frame->duration;
+  output_frame->interlace_mode = input_frame->interlace_mode;
+  output_frame->timecode = input_frame->timecode;
+  
   tmp_ctx = cnv->first_context;
 
+  
   while(tmp_ctx)
     {
     tmp_ctx->func(tmp_ctx);
     tmp_ctx = tmp_ctx->next;
     }
 
-  output_frame->time_scaled = input_frame->time_scaled;
-  output_frame->duration_scaled = input_frame->duration_scaled;
   }
