@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  videoformat.c
-
-  Copyright (c) 2001-2005 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <string.h>
 #include <stdio.h>
@@ -55,7 +57,7 @@ void gavl_video_format_dump(const gavl_video_format_t * format)
     }
   else
     {
-    fprintf(stderr, "Still image\n");
+    fprintf(stderr, "  Still image\n");
     }
   fprintf(stderr, "  Interlace mode:   %s\n", gavl_interlace_mode_to_string(format->interlace_mode));  
 
@@ -63,6 +65,14 @@ void gavl_video_format_dump(const gavl_video_format_t * format)
     {
     fprintf(stderr, "  Chroma placement: %s\n", gavl_chroma_placement_to_string(format->chroma_placement));
     }
+  if(format->timecode_format.int_framerate)
+    {
+    fprintf(stderr, "  Timecode framerate: %d\n", format->timecode_format.int_framerate);
+    fprintf(stderr, "  Timecode flags:     ");
+    if(format->timecode_format.flags & GAVL_TIMECODE_DROP_FRAME)
+      fprintf(stderr, "Drop");
+    fprintf(stderr, "\n");
+    }
   }
 
 /* We always enlarge the image */
@@ -93,7 +103,7 @@ void gavl_video_format_fit_to_source(gavl_video_format_t * dst,
     }
   }
 
-typedef struct
+typedef const struct
   {
   gavl_interlace_mode_t mode;
   char * name;
@@ -101,12 +111,13 @@ typedef struct
 
 interlace_mode_tab_t interlace_mode_tab[] =
   {
-    { GAVL_INTERLACE_NONE, "None (Progressive)" },
-    { GAVL_INTERLACE_TOP_FIRST, "Top field first" },
-    { GAVL_INTERLACE_BOTTOM_FIRST, "Bottom field first" }
+    { GAVL_INTERLACE_NONE,         "None (Progressive)" },
+    { GAVL_INTERLACE_TOP_FIRST,    "Top field first" },
+    { GAVL_INTERLACE_BOTTOM_FIRST, "Bottom field first" },
+    { GAVL_INTERLACE_MIXED,        "Mixed" }
   };
 
-static int num_interlace_modes = sizeof(interlace_mode_tab)/sizeof(interlace_mode_tab[0]);
+static const int num_interlace_modes = sizeof(interlace_mode_tab)/sizeof(interlace_mode_tab[0]);
 
 const char * gavl_interlace_mode_to_string(gavl_interlace_mode_t mode)
   {
@@ -126,14 +137,14 @@ typedef struct
   char * name;
   } chroma_placement_tab_t;
 
-chroma_placement_tab_t chroma_placement_tab[] =
+const chroma_placement_tab_t chroma_placement_tab[] =
   {
     { GAVL_CHROMA_PLACEMENT_DEFAULT, "MPEG-1/JPEG" },
     { GAVL_CHROMA_PLACEMENT_MPEG2, "MPEG-2" },
     { GAVL_CHROMA_PLACEMENT_DVPAL, "DV PAL" }
   };
 
-static int num_chroma_placements = sizeof(chroma_placement_tab)/sizeof(chroma_placement_tab_t);
+static const int num_chroma_placements = sizeof(chroma_placement_tab)/sizeof(chroma_placement_tab_t);
 
 const char * gavl_chroma_placement_to_string(gavl_chroma_placement_t mode)
   {
@@ -207,3 +218,9 @@ void gavl_video_format_get_chroma_offset(const gavl_video_format_t * format,
     }
   
   }
+
+int gavl_video_formats_equal(const gavl_video_format_t * format_1,
+                              const gavl_video_format_t * format_2)
+  {
+  return !memcmp(format_1, format_2, sizeof(*format_1));
+  }
