@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  videoframe.c
-
-  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <gavl.h>
 
@@ -41,7 +43,7 @@ void * memalign (size_t align, size_t size);
 #define memalign(align,size) malloc (size)
 #endif
 
-#define ALIGNMENT_BYTES 8
+#define ALIGNMENT_BYTES 16
 #define ALIGN(a) a=((a+ALIGNMENT_BYTES-1)/ALIGNMENT_BYTES)*ALIGNMENT_BYTES
 
 static void video_frame_alloc(gavl_video_frame_t * ret,
@@ -49,6 +51,15 @@ static void video_frame_alloc(gavl_video_frame_t * ret,
   {
   switch(format->pixelformat)
     {
+    case GAVL_GRAY_8:
+      ret->strides[0] = format->frame_width;
+      if(align)
+        ALIGN(ret->strides[0]);
+      ret->planes[0] = memalign(ALIGNMENT_BYTES,
+                                ret->strides[0] * format->frame_height);
+      break;
+    case GAVL_GRAYA_16:
+    case GAVL_GRAY_16:
     case GAVL_RGB_15:
     case GAVL_BGR_15:
     case GAVL_RGB_16:
@@ -70,6 +81,7 @@ static void video_frame_alloc(gavl_video_frame_t * ret,
     case GAVL_RGB_32:
     case GAVL_BGR_32:
     case GAVL_YUVA_32:
+    case GAVL_GRAYA_32:
       ret->strides[0] = format->frame_width*4;
       if(align)
         ALIGN(ret->strides[0]);
@@ -91,13 +103,30 @@ static void video_frame_alloc(gavl_video_frame_t * ret,
                              ret->strides[0] * format->frame_height);
       break;
     case GAVL_RGBA_64:
+    case GAVL_YUVA_64:
       ret->strides[0] = format->frame_width*8;
       if(align)
         ALIGN(ret->strides[0]);
       ret->planes[0] = memalign(ALIGNMENT_BYTES,
                              ret->strides[0] * format->frame_height);
       break;
+    case GAVL_GRAY_FLOAT:
+      ret->strides[0] = format->frame_width*sizeof(float);
+      if(align)
+        ALIGN(ret->strides[0]);
+      ret->planes[0] = memalign(ALIGNMENT_BYTES,
+                                ret->strides[0] * format->frame_height);
+      break;
+    case GAVL_GRAYA_FLOAT:
+      ret->strides[0] = 2*format->frame_width*sizeof(float);
+      if(align)
+        ALIGN(ret->strides[0]);
+      ret->planes[0] = memalign(ALIGNMENT_BYTES,
+                                ret->strides[0] * format->frame_height);
+      break;
+
     case GAVL_RGB_FLOAT:
+    case GAVL_YUV_FLOAT:
       ret->strides[0] = format->frame_width*3*sizeof(float);
       if(align)
         ALIGN(ret->strides[0]);
@@ -105,6 +134,7 @@ static void video_frame_alloc(gavl_video_frame_t * ret,
                              ret->strides[0] * format->frame_height);
       break;
     case GAVL_RGBA_FLOAT:
+    case GAVL_YUVA_FLOAT:
       ret->strides[0] = format->frame_width*4*sizeof(float);
       if(align)
         ALIGN(ret->strides[0]);
@@ -278,14 +308,16 @@ gavl_video_frame_t * gavl_video_frame_create(const gavl_video_format_t * format)
   gavl_video_frame_t * ret = calloc(1, sizeof(gavl_video_frame_t));
   if(format)
     video_frame_alloc(ret, format, 1);
+  ret->timecode = GAVL_TIMECODE_INVALID_MASK;
   return ret;
   }
 
-gavl_video_frame_t * gavl_video_frame_create_nopadd(const gavl_video_format_t * format)
+gavl_video_frame_t * gavl_video_frame_create_nopad(const gavl_video_format_t * format)
   {
   gavl_video_frame_t * ret = calloc(1, sizeof(gavl_video_frame_t));
   if(format)
     video_frame_alloc(ret, format, 0);
+  ret->timecode = GAVL_TIMECODE_INVALID_MASK;
   return ret;
   }
 
@@ -301,8 +333,8 @@ void gavl_video_frame_null(gavl_video_frame_t* frame)
   frame->planes[0] = (uint8_t*)0;
   }
 
-void gavl_video_frame_clear(gavl_video_frame_t * frame,
-                            gavl_video_format_t * format)
+void gavl_video_frame_clear_mask(gavl_video_frame_t * frame,
+                                 const gavl_video_format_t * format, int mask)
   {
   int i, j;
   uint16_t * ptr_16;
@@ -312,6 +344,7 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
   uint8_t * line_start;
   uint8_t * line_start_u;
   uint8_t * line_start_v;
+  int bytes;
   
   switch(format->pixelformat)
     {
@@ -325,9 +358,20 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
     case GAVL_BGR_32:
     case GAVL_RGB_48:
     case GAVL_RGB_FLOAT:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]);
+    case GAVL_GRAY_8:
+    case GAVL_GRAY_16:
+    case GAVL_GRAY_FLOAT:
+    case GAVL_YUV_FLOAT:
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width * gavl_pixelformat_bytes_per_pixel(format->pixelformat);
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
       break;
     case GAVL_RGBA_32:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
       for(i = 0; i < format->frame_height; i++)
         {
         for(j = 0; j < format->frame_width; j++)
@@ -339,7 +383,21 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
           }
         }
       break;
+    case GAVL_GRAYA_16:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
+      for(i = 0; i < format->frame_height; i++)
+        {
+        for(j = 0; j < format->frame_width; j++)
+          {
+          frame->planes[0][2*j + i*frame->strides[0]]   = 0x00; /* Gray */
+          frame->planes[0][2*j + i*frame->strides[0]+1] = 0xFF; /* A    */
+          }
+        }
+      break;
     case GAVL_YUVA_32:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
       for(i = 0; i < format->frame_height; i++)
         {
         for(j = 0; j < format->frame_width; j++)
@@ -352,6 +410,9 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         }
       break;
     case GAVL_RGBA_64:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
+      
       line_start = frame->planes[0];
       
       for(i = 0; i < format->frame_height; i++)
@@ -367,8 +428,49 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         line_start += frame->strides[0];
         }
       
+      break;
+    case GAVL_GRAYA_32:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
+      
+      line_start = frame->planes[0];
+      
+      for(i = 0; i < format->frame_height; i++)
+        {
+        ptr_16 = (uint16_t*)line_start;
+        for(j = 0; j < format->frame_width; j++)
+          {
+          *(ptr_16++) = 0x0000;
+          *(ptr_16++) = 0xFFFF;
+          }
+        line_start += frame->strides[0];
+        }
+      
+      break;
+    case GAVL_YUVA_64:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
+      
+      line_start = frame->planes[0];
+      
+      for(i = 0; i < format->frame_height; i++)
+        {
+        ptr_16 = (uint16_t*)line_start;
+        for(j = 0; j < format->frame_width; j++)
+          {
+          *(ptr_16++) = 0x0000;
+          *(ptr_16++) = 0x8000;
+          *(ptr_16++) = 0x8000;
+          *(ptr_16++) = 0xFFFF;
+          }
+        line_start += frame->strides[0];
+        }
+      
       break;
     case GAVL_RGBA_FLOAT:
+    case GAVL_YUVA_FLOAT:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
       line_start = frame->planes[0];
       
       for(i = 0; i < format->frame_height; i++)
@@ -384,7 +486,25 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         line_start += frame->strides[0];
         }
       break;
+    case GAVL_GRAYA_FLOAT:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
+      line_start = frame->planes[0];
+      
+      for(i = 0; i < format->frame_height; i++)
+        {
+        ptr_float = (float*)line_start;
+        for(j = 0; j < format->frame_width; j++)
+          {
+          *(ptr_float++) = 0.0;
+          *(ptr_float++) = 1.0;
+          }
+        line_start += frame->strides[0];
+        }
+      break;
     case GAVL_YUY2:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
       for(i = 0; i < format->frame_height; i++)
         {
         for(j = 0; j < format->frame_width; j++)
@@ -395,6 +515,8 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         }
       break;
     case GAVL_UYVY:
+      if(!(mask & CLEAR_MASK_PLANE_0))
+        break;
       for(i = 0; i < format->frame_height; i++)
         {
         for(j = 0; j < format->frame_width; j++)
@@ -405,7 +527,16 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         }
       break;
     case GAVL_YUV_444_P_16:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]); /* Y */
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width * 2;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      
+      if(!(mask & (CLEAR_MASK_PLANE_1 | CLEAR_MASK_PLANE_2)))
+        break;
+      
       line_start_u = frame->planes[1];
       line_start_v = frame->planes[2];
       for(i = 0; i < format->frame_height; i++)
@@ -413,17 +544,37 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         ptr_16_u = (uint16_t*)line_start_u;
         ptr_16_v = (uint16_t*)line_start_v;
 
-        for(j = 0; j < format->frame_width; j++)
+        if(mask & CLEAR_MASK_PLANE_1)
           {
-          *(ptr_16_u++) = 0x8000;
-          *(ptr_16_v++) = 0x8000;
+          for(j = 0; j < format->frame_width; j++)
+            {
+            *(ptr_16_u++) = 0x8000;
+            }
           }
+        
+        if(mask & CLEAR_MASK_PLANE_2)
+          {
+          for(j = 0; j < format->frame_width; j++)
+            {
+            *(ptr_16_v++) = 0x8000;
+            }
+          }
+        
         line_start_u += frame->strides[1];
         line_start_v += frame->strides[2];
         }
       break;
     case GAVL_YUV_422_P_16:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]); /* Y */
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width * 2;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      
+      if(!(mask & (CLEAR_MASK_PLANE_1 | CLEAR_MASK_PLANE_2)))
+        break;
+      
       line_start_u = frame->planes[1];
       line_start_v = frame->planes[2];
       for(i = 0; i < format->frame_height; i++)
@@ -431,43 +582,142 @@ void gavl_video_frame_clear(gavl_video_frame_t * frame,
         ptr_16_u = (uint16_t*)line_start_u;
         ptr_16_v = (uint16_t*)line_start_v;
 
-        for(j = 0; j < format->frame_width/2; j++)
+        if(mask & CLEAR_MASK_PLANE_1)
           {
-          *(ptr_16_u++) = 0x8000;
-          *(ptr_16_v++) = 0x8000;
+          for(j = 0; j < format->frame_width/2; j++)
+            {
+            *(ptr_16_u++) = 0x8000;
+            }
           }
+        if(mask & CLEAR_MASK_PLANE_2)
+          {
+          for(j = 0; j < format->frame_width/2; j++)
+            {
+            *(ptr_16_v++) = 0x8000;
+            }
+          }
+        
         line_start_u += frame->strides[1];
         line_start_v += frame->strides[2];
         }
       break;
     case GAVL_YUV_420_P:
     case GAVL_YUVJ_420_P:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]);
-      memset(frame->planes[1], 0x80, (format->frame_height * frame->strides[1])/2);
-      memset(frame->planes[2], 0x80, (format->frame_height * frame->strides[2])/2);
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_1)
+        {
+        bytes = format->frame_width / 2;
+        for(i = 0; i < format->frame_height / 2; i++)
+          memset(frame->planes[1] + i * frame->strides[1], 0x80, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_2)
+        {
+        bytes = format->frame_width / 2;
+        for(i = 0; i < format->frame_height / 2; i++)
+          memset(frame->planes[2] + i * frame->strides[2], 0x80, bytes);
+        }
       break;
     case GAVL_YUV_410_P:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]);
-      memset(frame->planes[1], 0x80, (format->frame_height * frame->strides[1])/4);
-      memset(frame->planes[2], 0x80, (format->frame_height * frame->strides[2])/4);
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_1)
+        {
+        bytes = format->frame_width / 4;
+        for(i = 0; i < format->frame_height / 4; i++)
+          memset(frame->planes[1] + i * frame->strides[1], 0x80, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_2)
+        {
+        bytes = format->frame_width / 4;
+        for(i = 0; i < format->frame_height / 4; i++)
+          memset(frame->planes[2] + i * frame->strides[2], 0x80, bytes);
+        }
+      break;
+    case GAVL_YUV_411_P:
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_1)
+        {
+        bytes = format->frame_width / 4;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[1] + i * frame->strides[1], 0x80, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_2)
+        {
+        bytes = format->frame_width / 4;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[2] + i * frame->strides[2], 0x80, bytes);
+        }
       break;
     case GAVL_YUV_422_P:
-    case GAVL_YUV_411_P:
-    case GAVL_YUV_444_P:
     case GAVL_YUVJ_422_P:
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_1)
+        {
+        bytes = format->frame_width / 2;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[1] + i * frame->strides[1], 0x80, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_2)
+        {
+        bytes = format->frame_width / 2;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[2] + i * frame->strides[2], 0x80, bytes);
+        }
+      break;
+    case GAVL_YUV_444_P:
     case GAVL_YUVJ_444_P:
-      memset(frame->planes[0], 0x00, format->frame_height * frame->strides[0]);
-      memset(frame->planes[1], 0x80, format->frame_height * frame->strides[1]);
-      memset(frame->planes[2], 0x80, format->frame_height * frame->strides[2]);
+      if(mask & CLEAR_MASK_PLANE_0)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[0] + i * frame->strides[0], 0x00, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_1)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[1] + i * frame->strides[1], 0x80, bytes);
+        }
+      if(mask & CLEAR_MASK_PLANE_2)
+        {
+        bytes = format->frame_width;
+        for(i = 0; i < format->frame_height; i++)
+          memset(frame->planes[2] + i * frame->strides[2], 0x80, bytes);
+        }
       break;
     case GAVL_PIXELFORMAT_NONE:
       break;
-      
     }
   }
 
+
+void gavl_video_frame_clear(gavl_video_frame_t * frame,
+                            const gavl_video_format_t * format)
+  {
+  gavl_video_frame_clear_mask(frame, format, CLEAR_MASK_ALL);  
+  }
+
 static void copy_plane(gavl_video_frame_t * dst,
-                       gavl_video_frame_t * src, int plane,
+                       const gavl_video_frame_t * src, int plane,
                        int bytes_per_line, int height)
   {
   int j;
@@ -489,9 +739,9 @@ static void copy_plane(gavl_video_frame_t * dst,
     }
   }
 
-void gavl_video_frame_copy_plane(gavl_video_format_t * format,
+void gavl_video_frame_copy_plane(const gavl_video_format_t * format,
                                  gavl_video_frame_t * dst,
-                                 gavl_video_frame_t * src, int plane)
+                                 const gavl_video_frame_t * src, int plane)
   {
   int bytes_per_line;
   int sub_h, sub_v;
@@ -513,9 +763,9 @@ void gavl_video_frame_copy_plane(gavl_video_format_t * format,
   copy_plane(dst, src, plane, bytes_per_line, height);
   }
 
-void gavl_video_frame_copy(gavl_video_format_t * format,
+void gavl_video_frame_copy(const gavl_video_format_t * format,
                            gavl_video_frame_t * dst,
-                           gavl_video_frame_t * src)
+                           const gavl_video_frame_t * src)
   {
   int i;
   int height;
@@ -548,6 +798,8 @@ void gavl_video_frame_copy(gavl_video_format_t * format,
   
   }
 
+
+
 static void flip_scanline_1(uint8_t * dst, uint8_t * src, int len)
   {
   int i;
@@ -577,6 +829,43 @@ static void flip_scanline_2(uint8_t * dst, uint8_t * src, int len)
   
   }
 
+static void flip_scanline_yuy2(uint8_t * dst, uint8_t * src, int len)
+  {
+  int i;
+  dst += 2*(len-1)-2;
+  
+  for(i = 0; i < len/2; i++)
+    {
+    dst[0] = src[2]; /* Y */
+    dst[1] = src[1]; /* U */
+    dst[2] = src[0]; /* Y */
+    dst[3] = src[3]; /* U */
+    
+    dst-=4;
+    src+=4;
+    }
+  
+  }
+
+static void flip_scanline_uyvy(uint8_t * dst, uint8_t * src, int len)
+  {
+  int i;
+  dst += 2*(len-1)-2;
+  
+  for(i = 0; i < len/2; i++)
+    {
+    dst[0] = src[0]; /* U */
+    dst[1] = src[3]; /* Y */
+    dst[2] = src[2]; /* V */
+    dst[3] = src[1]; /* Y */
+    
+    dst-=4;
+    src+=4;
+    }
+  
+  }
+
+
 static void flip_scanline_3(uint8_t * dst, uint8_t * src, int len)
   {
   int i;
@@ -695,10 +984,10 @@ static flip_scanline_func find_flip_scanline_func(gavl_pixelformat_t csp)
     case GAVL_BGR_15:
     case GAVL_RGB_16:
     case GAVL_BGR_16:
-    case GAVL_YUY2:
-    case GAVL_UYVY:
     case GAVL_YUV_444_P_16:
     case GAVL_YUV_422_P_16:
+    case GAVL_GRAYA_16:
+    case GAVL_GRAY_16:
       return flip_scanline_2;
       break;
     case GAVL_RGB_24:
@@ -709,18 +998,24 @@ static flip_scanline_func find_flip_scanline_func(gavl_pixelformat_t csp)
     case GAVL_BGR_32:
     case GAVL_RGBA_32:
     case GAVL_YUVA_32:
+    case GAVL_GRAYA_32:
+    case GAVL_GRAY_FLOAT:
       return flip_scanline_4;
       break;
     case GAVL_RGB_48:
       return flip_scanline_6;
       break;
     case GAVL_RGBA_64:
+    case GAVL_YUVA_64:
+    case GAVL_GRAYA_FLOAT:
       return flip_scanline_8;
       break;
     case GAVL_RGB_FLOAT:
+    case GAVL_YUV_FLOAT:
       return flip_scanline_12;
       break;
     case GAVL_RGBA_FLOAT:
+    case GAVL_YUVA_FLOAT:
       return flip_scanline_16;
       break;
     case GAVL_YUV_420_P:
@@ -731,22 +1026,29 @@ static flip_scanline_func find_flip_scanline_func(gavl_pixelformat_t csp)
     case GAVL_YUVJ_420_P:
     case GAVL_YUVJ_422_P:
     case GAVL_YUVJ_444_P:
+    case GAVL_GRAY_8:
       return flip_scanline_1;
       break;
+    case GAVL_YUY2:
+      return flip_scanline_yuy2;
+      break;
+    case GAVL_UYVY:
+      return flip_scanline_uyvy;
+      break;
     case GAVL_PIXELFORMAT_NONE:
       break;
     }
   return (flip_scanline_func)0;
   }
 
-void gavl_video_frame_copy_flip_x(gavl_video_format_t * format,
+void gavl_video_frame_copy_flip_x(const gavl_video_format_t * format,
                                   gavl_video_frame_t * dst,
-                                  gavl_video_frame_t * src)
+                                  const gavl_video_frame_t * src)
   {
   uint8_t * src_ptr;
   uint8_t * dst_ptr;
   
-  int i, j;
+  int i, j, jmax, width;
   int sub_h, sub_v;
   int planes;
   flip_scanline_func func;
@@ -756,29 +1058,36 @@ void gavl_video_frame_copy_flip_x(gavl_video_format_t * format,
   
   sub_h = 1;
   sub_v = 1;
+
+  jmax = format->image_height;
+  width = format->image_width;
+
+  gavl_pixelformat_chroma_sub(format->pixelformat, &sub_h, &sub_v);
   
   for(i = 0; i < planes; i++)
     {
-    if(i)
-      gavl_pixelformat_chroma_sub(format->pixelformat, &sub_h, &sub_v);
-
     src_ptr = src->planes[i];
     dst_ptr = dst->planes[i];
             
-    for(j = 0; j < format->image_height / sub_v; j++)
+    for(j = 0; j < jmax; j++)
       {
-      func(dst_ptr, src_ptr, format->image_width / sub_h);
-
+      func(dst_ptr, src_ptr, width);
+      
       src_ptr += src->strides[i];
       dst_ptr += dst->strides[i];
       }
+    if(!i)
+      {
+      jmax /= sub_v;
+      width /= sub_h;
+      }
     }
   
   }
 
-void gavl_video_frame_copy_flip_y(gavl_video_format_t * format,
+void gavl_video_frame_copy_flip_y(const gavl_video_format_t * format,
                                   gavl_video_frame_t * dst,
-                                  gavl_video_frame_t * src)
+                                  const gavl_video_frame_t * src)
   {
   uint8_t * src_ptr;
   uint8_t * dst_ptr;
@@ -818,9 +1127,9 @@ void gavl_video_frame_copy_flip_y(gavl_video_format_t * format,
 
   }
 
-void gavl_video_frame_copy_flip_xy(gavl_video_format_t * format,
-                                  gavl_video_frame_t * dst,
-                                  gavl_video_frame_t * src)
+void gavl_video_frame_copy_flip_xy(const gavl_video_format_t * format,
+                                   gavl_video_frame_t * dst,
+                                   const gavl_video_frame_t * src)
   {
   uint8_t * src_ptr;
   uint8_t * dst_ptr;
@@ -860,7 +1169,7 @@ void gavl_video_frame_copy_flip_xy(gavl_video_format_t * format,
 
 
 void gavl_video_frame_dump(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            const char * namebase)
   {
   char * filename;
@@ -905,7 +1214,7 @@ void gavl_video_frame_dump(gavl_video_frame_t * frame,
   }
 
 void gavl_video_frame_get_subframe(gavl_pixelformat_t pixelformat,
-                                   gavl_video_frame_t * src,
+                                   const gavl_video_frame_t * src,
                                    gavl_video_frame_t * dst,
                                    gavl_rectangle_i_t * src_rect)
   {
@@ -940,7 +1249,7 @@ void gavl_video_frame_get_subframe(gavl_pixelformat_t pixelformat,
   }
 
 void gavl_video_frame_get_field(gavl_pixelformat_t pixelformat,
-                                gavl_video_frame_t * src,
+                                const gavl_video_frame_t * src,
                                 gavl_video_frame_t * dst,
                                 int field)
   {
@@ -983,8 +1292,17 @@ void gavl_video_frame_get_field(gavl_pixelformat_t pixelformat,
     dst_start += frame->strides[0];                 \
     }
 
+static void fill_8_packed(gavl_video_frame_t * frame,
+                          const gavl_video_format_t * format,
+                          uint8_t color)
+  {
+  FILL_FUNC_HEAD_PACKED(uint8_t);
+  *dst = color;
+  FILL_FUNC_TAIL_PACKED(1);
+  }
+      
 static void fill_16_packed(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint16_t color)
   {
   FILL_FUNC_HEAD_PACKED(uint16_t);
@@ -993,7 +1311,7 @@ static void fill_16_packed(gavl_video_frame_t * frame,
   }
 
 static void fill_24_packed(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint8_t * color)
   {
   FILL_FUNC_HEAD_PACKED(uint8_t);
@@ -1004,7 +1322,7 @@ static void fill_24_packed(gavl_video_frame_t * frame,
   }
 
 static void fill_32_packed(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint8_t * _color)
   {
   uint32_t * color = (uint32_t*)_color;
@@ -1014,7 +1332,7 @@ static void fill_32_packed(gavl_video_frame_t * frame,
   }
 
 static void fill_32_packed_422(gavl_video_frame_t * frame,
-                               gavl_video_format_t * format,
+                               const gavl_video_format_t * format,
                                uint8_t * _color)
   {
   uint32_t * color = (uint32_t*)_color;
@@ -1025,7 +1343,7 @@ static void fill_32_packed_422(gavl_video_frame_t * frame,
 
 
 static void fill_48_packed(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint16_t * color)
   {
   FILL_FUNC_HEAD_PACKED(uint16_t);
@@ -1034,7 +1352,7 @@ static void fill_48_packed(gavl_video_frame_t * frame,
   }
 
 static void fill_64_packed(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint16_t * color)
   {
   FILL_FUNC_HEAD_PACKED(uint16_t);
@@ -1042,8 +1360,26 @@ static void fill_64_packed(gavl_video_frame_t * frame,
   FILL_FUNC_TAIL_PACKED(4);
   }
 
+static void fill_float_gray(gavl_video_frame_t * frame,
+                           const gavl_video_format_t * format,
+                           float * color)
+  {
+  FILL_FUNC_HEAD_PACKED(float);
+  gavl_memcpy(dst, color, sizeof(*dst));
+  FILL_FUNC_TAIL_PACKED(1);
+  }
+
+static void fill_float_graya(gavl_video_frame_t * frame,
+                           const gavl_video_format_t * format,
+                           float * color)
+  {
+  FILL_FUNC_HEAD_PACKED(float);
+  gavl_memcpy(dst, color, 2*sizeof(*dst));
+  FILL_FUNC_TAIL_PACKED(2);
+  }
+
 static void fill_float_rgb(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            float * color)
   {
   FILL_FUNC_HEAD_PACKED(float);
@@ -1052,7 +1388,7 @@ static void fill_float_rgb(gavl_video_frame_t * frame,
   }
 
 static void fill_float_rgba(gavl_video_frame_t * frame,
-                            gavl_video_format_t * format,
+                            const gavl_video_format_t * format,
                             float * color)
   {
   FILL_FUNC_HEAD_PACKED(float);
@@ -1079,7 +1415,7 @@ static void fill_float_rgba(gavl_video_frame_t * frame,
 
 
 static void fill_planar_8(gavl_video_frame_t * frame,
-                          gavl_video_format_t * format,
+                          const gavl_video_format_t * format,
                           uint8_t * color)
   {
   int i, imax;
@@ -1113,7 +1449,7 @@ static void fill_planar_8(gavl_video_frame_t * frame,
   }
 
 static void fill_planar_16(gavl_video_frame_t * frame,
-                           gavl_video_format_t * format,
+                           const gavl_video_format_t * format,
                            uint16_t * color)
   {
   int i, j, imax, jmax;
@@ -1167,18 +1503,56 @@ static void fill_planar_16(gavl_video_frame_t * frame,
 
 
 void gavl_video_frame_fill(gavl_video_frame_t * frame,
-                            gavl_video_format_t * format,
-                            float * color)
+                           const gavl_video_format_t * format,
+                           float * color)
   {
   INIT_RGB_FLOAT_TO_YUV
   uint16_t packed_16;
   uint8_t  packed_32[4];
   uint16_t packed_64[4];
+  float color_float[2];
 
   gavl_init_memcpy();
   
   switch(format->pixelformat)
     {
+    case GAVL_GRAY_8:
+      RGB_FLOAT_TO_YUVJ_8(color[0], color[1], color[2], packed_32[0],
+                          packed_32[1], packed_32[2]);
+      fill_8_packed(frame, format, packed_32[0]);
+      break;
+    case GAVL_GRAYA_16:
+      RGB_FLOAT_TO_YUVJ_8(color[0], color[1], color[2], packed_32[0],
+                          packed_32[1], packed_32[2]);
+      /* This overwrites packed_32[1] from the lone before */
+      RGB_FLOAT_TO_8(color[3], packed_32[1]);
+#ifndef WORDS_BIGENDIAN
+      packed_16 = (packed_32[1] << 8) | packed_32[0];
+#else
+      packed_16 = (packed_32[0] << 8) | packed_32[1];
+#endif 
+      fill_16_packed(frame, format, packed_16);
+      break;
+    case GAVL_GRAY_16:
+      RGB_FLOAT_TO_YJ_16(color[0], color[1], color[2], packed_16);
+      fill_16_packed(frame, format, packed_16);
+      break;
+    case GAVL_GRAYA_32:
+      RGB_FLOAT_TO_YJ_16(color[0], color[1], color[2], packed_64[0]);
+      RGB_FLOAT_TO_16(color[3], packed_64[1]);
+#ifndef WORDS_BIGENDIAN
+      packed_32[0] = packed_64[0] & 0xff;
+      packed_32[1] = packed_64[0] >> 8;
+      packed_32[2] = packed_64[1] & 0xff;
+      packed_32[3] = packed_64[1] >> 8;
+#else
+      packed_32[1] = packed_64[0] & 0xff;
+      packed_32[0] = packed_64[0] >> 8;
+      packed_32[3] = packed_64[1] & 0xff;
+      packed_32[2] = packed_64[1] >> 8;
+#endif 
+      fill_32_packed(frame, format, packed_32);
+      break;
     case GAVL_RGB_15:
       RGB_FLOAT_TO_8(color[0], packed_32[0]);
       RGB_FLOAT_TO_8(color[1], packed_32[1]);
@@ -1257,6 +1631,32 @@ void gavl_video_frame_fill(gavl_video_frame_t * frame,
       RGB_FLOAT_TO_16(color[3], packed_64[3]);
       fill_64_packed(frame, format, packed_64);
       break;
+    case GAVL_YUVA_64:
+      RGB_FLOAT_TO_YUV_16(color[0], color[1], color[2], packed_64[0],
+                          packed_64[1], packed_64[2]);
+      RGB_FLOAT_TO_16(color[3], packed_64[3]);
+      fill_64_packed(frame, format, packed_64);
+      break;
+    case GAVL_GRAY_FLOAT:
+      RGB_FLOAT_TO_Y_FLOAT(color[0], color[1], color[2], color_float[0]);
+      fill_float_gray(frame, format, color_float);
+      break;
+    case GAVL_GRAYA_FLOAT:
+      RGB_FLOAT_TO_Y_FLOAT(color[0], color[1], color[2], color_float[0]);
+      color_float[1] = color[4];
+      fill_float_graya(frame, format, color_float);
+      break;
+    case GAVL_YUV_FLOAT:
+      RGB_FLOAT_TO_YUV_FLOAT(color[0], color[1], color[2],
+                             color_float[0], color_float[1], color_float[2]);
+      fill_float_rgb(frame, format, color_float);
+      break;
+    case GAVL_YUVA_FLOAT:
+      RGB_FLOAT_TO_YUV_FLOAT(color[0], color[1], color[2],
+                             color_float[0], color_float[1], color_float[2]);
+      color_float[3] = color[3];
+      fill_float_rgba(frame, format, color);
+      break;
     case GAVL_RGB_FLOAT:
       fill_float_rgb(frame, format, color);
       break;
@@ -1306,3 +1706,57 @@ void gavl_video_frame_fill(gavl_video_frame_t * frame,
       return;
     }
   }
+
+void gavl_video_frame_copy_metadata(gavl_video_frame_t * dst,
+                                    const gavl_video_frame_t * src)
+  {
+  dst->timestamp       = src->timestamp;
+  dst->duration        = src->duration;
+  dst->timecode        = src->timecode;
+  dst->interlace_mode  = src->interlace_mode;
+  }
+
+void gavl_video_frame_set_strides(gavl_video_frame_t * frame,
+                                  const gavl_video_format_t * format)
+  {
+  int i;
+  int bytes_per_line;
+  int sub_h, sub_v;
+  int num_planes = gavl_pixelformat_num_planes(format->pixelformat);
+  bytes_per_line = gavl_pixelformat_is_planar(format->pixelformat) ?
+    format->frame_width * gavl_pixelformat_bytes_per_component(format->pixelformat) :
+    format->frame_width * gavl_pixelformat_bytes_per_pixel(format->pixelformat);
+
+  gavl_pixelformat_chroma_sub(format->pixelformat, &sub_h, &sub_v);
+
+  for(i = 0; i < num_planes; i++)
+    {
+    frame->strides[i] = bytes_per_line;
+    if(i)
+      frame->strides[i] /= sub_h;
+    }
+  }
+  
+void gavl_video_frame_set_planes(gavl_video_frame_t * frame,
+                                 const gavl_video_format_t * format,
+                                 uint8_t * buffer)
+  {
+  int i;
+  int sub_h, sub_v;
+  int advance;
+  int num_planes = gavl_pixelformat_num_planes(format->pixelformat);
+  if(!frame->strides[0])
+    gavl_video_frame_set_strides(frame, format);
+
+  gavl_pixelformat_chroma_sub(format->pixelformat, &sub_h, &sub_v);
+
+  for(i = 0; i < num_planes; i++)
+    {
+    frame->planes[i] = buffer;
+    advance = frame->strides[i] * format->frame_height;
+    if(i)
+      advance /= sub_v;
+    buffer += advance;
+    }
+  
+  }
