@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  volume.c
-
-  Copyright (c) 2004 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #include <stdlib.h>
 #include <math.h>
@@ -41,6 +43,7 @@ static void set_factor_i(gavl_volume_control_t * v)
       break;
     case GAVL_SAMPLE_NONE:
     case GAVL_SAMPLE_FLOAT:
+    case GAVL_SAMPLE_DOUBLE:
       break;
     }
   }
@@ -104,7 +107,7 @@ void gavl_volume_control_destroy(gavl_volume_control_t * v)
 /* Set format: can be called multiple times with one instance */
 
 void gavl_volume_control_set_format(gavl_volume_control_t * v,
-                                    gavl_audio_format_t * format)
+                                    const gavl_audio_format_t * format)
   {
   gavl_volume_funcs_t * funcs;
 
@@ -137,6 +140,9 @@ void gavl_volume_control_set_format(gavl_volume_control_t * v,
     case GAVL_SAMPLE_FLOAT:
       v->set_volume_channel = funcs->set_volume_float;
       break;
+    case GAVL_SAMPLE_DOUBLE:
+      v->set_volume_channel = funcs->set_volume_double;
+      break;
 
     case GAVL_SAMPLE_NONE:
       break;
