@@ -1,25 +1,29 @@
 /*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
-  audio.h
-
-  Copyright (c) 2003 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
-
-/* Private structures for the video converter */
+/* Private structures for the audio converter */
 
 #include <gavl.h>
+#include <samplerate.h>
+
 #include "config.h"
 
 struct gavl_audio_options_s
@@ -52,9 +56,17 @@ typedef struct gavl_samplerate_converter_s gavl_samplerate_converter_t;
 
 typedef struct gavl_audio_dither_context_s gavl_audio_dither_context_t;
 
+struct gavl_samplerate_converter_s
+  {
+  int num_resamplers;
+  SRC_STATE ** resamplers;
+  SRC_DATA data;
+  double ratio;
+  };
+
 struct gavl_audio_convert_context_s
   {
-  gavl_audio_frame_t * input_frame;
+  const gavl_audio_frame_t * input_frame;
   gavl_audio_frame_t * output_frame;
 
   gavl_audio_format_t input_format;
