@@ -1,22 +1,33 @@
 /*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
  
-  gavltime.h
- 
-  Copyright (c) 2001-2002 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
- 
-  http://gmerlin.sourceforge.net
- 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
- 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
- 
-*****************************************************************/
+#ifndef GAVLTIME_H_INCLUDED
+#define GAVLTIME_H_INCLUDED
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#pragma GCC visibility push(default)
+  
 /*! \defgroup time Time
  */
 
@@ -44,7 +55,6 @@ typedef int64_t gavl_time_t;
 
 /*! \ingroup time
  * \brief Convert a number of samples to a time for a given samplerate
- * \todo Write this as an overflow save function
  */
 
 gavl_time_t gavl_samples_to_time(int samplerate, int64_t samples);
@@ -54,14 +64,12 @@ gavl_time_t gavl_samples_to_time(int samplerate, int64_t samples);
  * \param samplerate Samplerate
  * \param time Time
  * \returns Number of audio samples
- * \todo Write this as an overflow save function
  */
 
 int64_t gavl_time_to_samples(int samplerate, gavl_time_t time);
 
 /*! \ingroup time
  * \brief Convert a number of video frames to a time for a given framerate
- * \todo Write this as an overflow save function
  */
 
 gavl_time_t gavl_frames_to_time(int rate_num, int rate_den, int64_t frames);
@@ -73,7 +81,6 @@ gavl_time_t gavl_frames_to_time(int rate_num, int rate_den, int64_t frames);
  * \param rate_den Denominator of the framerate
  * \param time Time
  * \returns Number of frames
- * \todo Write this as an overflow save function
  */
 
 int64_t gavl_time_to_frames(int rate_num, int rate_den, gavl_time_t time); 
@@ -83,7 +90,6 @@ int64_t gavl_time_to_frames(int rate_num, int rate_den, gavl_time_t time);
  * \param scale Time scale
  * \param time Time scaled by \ref GAVL_TIME_SCALE
  * \returns Time scaled by scale
- * \todo Write this as an overflow save function
  */
 
 int64_t gavl_time_scale(int scale, gavl_time_t time);
@@ -93,7 +99,6 @@ int64_t gavl_time_scale(int scale, gavl_time_t time);
  * \param scale Time scale
  * \param time Time scaled by scale
  * \returns Time scaled by \ref GAVL_TIME_SCALE
- * \todo Write this as an overflow save function
  */
 
 gavl_time_t gavl_time_unscale(int scale, int64_t time);
@@ -104,7 +109,6 @@ gavl_time_t gavl_time_unscale(int scale, int64_t time);
  * \param scale2 New time base
  * \param time Time scaled by scale1
  * \returns Time scaled by scale2
- * \todo Write this as an overflow save function
  */
 
 int64_t gavl_time_rescale(int scale1, int scale2, int64_t time);
@@ -123,7 +127,7 @@ int64_t gavl_time_rescale(int scale1, int scale2, int64_t time);
  * \returns Seconds as double
  */
 
-#define gavl_time_to_seconds(t) ((double)t/(double)(GAVL_TIME_SCALE))
+#define gavl_time_to_seconds(t) ((double)(t)/(double)(GAVL_TIME_SCALE))
 
 /*! \ingroup time
  * \brief Sleep for a specified time
@@ -220,3 +224,12 @@ gavl_time_t gavl_timer_get(gavl_timer_t * timer);
  */
 
 void gavl_timer_set(gavl_timer_t * timer, gavl_time_t t);
+
+#pragma GCC visibility pop
+  
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* GAVLTIME_H_INCLUDED */
+
