@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  video.h
-
-  Copyright (c) 2001-2002 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 #ifndef _GAVL_VIDEO_H_
 #define _GAVL_VIDEO_H_
@@ -48,8 +50,6 @@ struct gavl_video_options_s
   gavl_deinterlace_mode_t deinterlace_mode;
   gavl_deinterlace_drop_mode_t deinterlace_drop_mode;
 
-  int deinterlace_force;
-  
   /* Background color (Floating point and 16 bit int)background_float[3]; */
   float background_float[3];
 
@@ -62,6 +62,9 @@ struct gavl_video_options_s
   int have_rectangles;
   
   int keep_aspect;
+  
+  gavl_downscale_filter_t downscale_filter;
+  float downscale_blur;
   };
 
 typedef struct gavl_video_convert_context_s gavl_video_convert_context_t;
@@ -70,7 +73,7 @@ typedef void (*gavl_video_func_t)(gavl_video_convert_context_t * ctx);
 
 struct gavl_video_convert_context_s
   {
-  gavl_video_frame_t * input_frame;
+  const gavl_video_frame_t * input_frame;
   gavl_video_frame_t * output_frame;
   gavl_video_options_t * options;
 
@@ -86,6 +89,9 @@ struct gavl_video_convert_context_s
 
 struct gavl_video_converter_s
   {
+  gavl_video_format_t input_format;
+  gavl_video_format_t output_format;
+  
   gavl_video_options_t options;
 
   gavl_video_convert_context_t * first_context;
@@ -115,4 +121,22 @@ int gavl_pixelformat_can_scale(gavl_pixelformat_t in_csp, gavl_pixelformat_t out
 gavl_pixelformat_t gavl_pixelformat_get_intermediate(gavl_pixelformat_t in_csp,
                                                    gavl_pixelformat_t out_csp);
 
+#define CLEAR_MASK_PLANE_0 (1<<0)
+#define CLEAR_MASK_PLANE_1 (1<<1)
+#define CLEAR_MASK_PLANE_2 (1<<2)
+#define CLEAR_MASK_PLANE_3 (1<<3)
+#define CLEAR_MASK_ALL (CLEAR_MASK_PLANE_0|\
+                        CLEAR_MASK_PLANE_1|\
+                        CLEAR_MASK_PLANE_2|\
+                        CLEAR_MASK_PLANE_3)
+
+void gavl_video_frame_clear_mask(gavl_video_frame_t * frame,
+                                 const gavl_video_format_t * format, int mask);
+
+void gavl_pixelformat_get_offset(gavl_pixelformat_t pixelformat,
+                                 int plane,
+                                 int * advance, int * offset);
+
+
+
 #endif // _GAVL_VIDEO_H_
