@@ -1,21 +1,23 @@
 /*****************************************************************
-
-  volume.h
-
-  Copyright (c) 2004 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
-
-  http://gmerlin.sourceforge.net
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
-
-*****************************************************************/
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 
 typedef struct
   {
@@ -34,13 +36,15 @@ typedef struct
 
   void (*set_volume_float)(gavl_volume_control_t * v, void * samples,
                          int num_samples);
+  void (*set_volume_double)(gavl_volume_control_t * v, void * samples,
+                            int num_samples);
   } gavl_volume_funcs_t;
 
 struct gavl_volume_control_s
   {
   gavl_audio_format_t format;
   
-  float factor_f;
+  double factor_f;
   int64_t factor_i;
   
   void (*set_volume)(gavl_volume_control_t * v,
