@@ -1,3 +1,23 @@
+/*****************************************************************
+ * gavl - a general purpose audio/video processing library
+ *
+ * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * gmerlin-general@lists.sourceforge.net
+ * http://gmerlin.sourceforge.net
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * *****************************************************************/
 #include <sys/time.h>
 #include <stdlib.h>
 #include <gavl.h>
@@ -6,39 +26,20 @@
 #include <string.h>
 
 #include <accel.h>
+#include "timeutils.h"
 
 #define NUM_CONVERSIONS 20
 
-static struct timeval time_before;
-static struct timeval time_after;
-
-static void timer_init()
-  {
-  gettimeofday(&time_before, (struct timezone*)0);
-  }
-
-static void timer_stop()
-  {
-  double before, after, diff;
-  
-  gettimeofday(&time_after, (struct timezone*)0);
-
-  before = time_before.tv_sec + time_before.tv_usec / 1.0e6;
-  after  = time_after.tv_sec  + time_after.tv_usec  / 1.0e6;
-
-/*   fprintf(stderr, "Before: %f After: %f\n", before, after); */
-    
-  diff = after - before;
-    
-  fprintf(stderr, "Made %d conversions, Time: %f (%f per conversion)\n",
-          NUM_CONVERSIONS, diff, diff/NUM_CONVERSIONS);
-  }
+//#define SCALE_MODE GAVL_SCALE_SINC_LANCZOS
+#define SCALE_MODE GAVL_SCALE_BILINEAR
 
 
 int main(int argc, char ** argv)
   {
   gavl_rectangle_f_t src_rect;
   gavl_rectangle_i_t dst_rect;
+
+  uint64_t t;
   
   int i, j, imax;
   gavl_video_scaler_t *scaler;
@@ -105,8 +106,10 @@ int main(int argc, char ** argv)
     fprintf(stderr, "C-Version:\n");
     
     gavl_video_options_set_defaults(opt);
-    gavl_video_options_set_scale_mode(opt, GAVL_SCALE_BILINEAR);
-    gavl_video_options_set_accel_flags(opt, GAVL_ACCEL_C);
+    gavl_video_options_set_scale_mode(opt, SCALE_MODE);
+    gavl_video_options_set_scale_order(opt, 5);
+    gavl_video_options_set_quality(opt, 3);
+    //    gavl_video_options_set_accel_flags(opt, GAVL_ACCEL_C);
     gavl_video_options_set_rectangles(opt, &src_rect, &dst_rect);
 
     if(gavl_video_scaler_init(scaler,
@@ -121,13 +124,16 @@ int main(int argc, char ** argv)
         {
         gavl_video_scaler_scale(scaler, frame, frame_1);
         }
-      timer_stop();
+      t = timer_stop();
+      fprintf(stderr, "Made %d conversions, Time: %e (%e per conversion)\n",
+              NUM_CONVERSIONS, (double)t, (double)t/NUM_CONVERSIONS);
       }
-#if 0
+#if 1
     fprintf(stderr, "MMX-Version:\n");
     gavl_video_options_set_defaults(opt);
-    gavl_video_options_set_scale_mode(opt, GAVL_SCALE_BILINEAR);
-    gavl_video_options_set_accel_flags(opt, GAVL_ACCEL_MMX);
+    gavl_video_options_set_scale_mode(opt, SCALE_MODE);
+    gavl_video_options_set_quality(opt, 2);
+    gavl_video_options_set_scale_order(opt, 5);
     gavl_video_options_set_rectangles(opt, &src_rect, &dst_rect);
         
     if(gavl_video_scaler_init(scaler,
@@ -142,29 +148,11 @@ int main(int argc, char ** argv)
         {
         gavl_video_scaler_scale(scaler, frame, frame_1);
         }
-      timer_stop();
+      t = timer_stop();
+      fprintf(stderr, "Made %d conversions, Time: %e (%e per conversion)\n",
+              NUM_CONVERSIONS, (double)t, (double)t/NUM_CONVERSIONS);
       }
 
-    fprintf(stderr, "MMXEXT-Version:\n");
-    gavl_video_options_set_defaults(opt);
-    gavl_video_options_set_scale_mode(opt, GAVL_SCALE_BILINEAR);
-    gavl_video_options_set_accel_flags(opt, GAVL_ACCEL_MMXEXT);
-    gavl_video_options_set_rectangles(opt, &src_rect, &dst_rect);
-        
-    if(gavl_video_scaler_init(scaler,
-                              &format, &format_1) < 0)  // int output_height
-      {
-      fprintf(stderr, "No scaling routine defined\n");
-      }
-    else
-      {
-      timer_init();
-      for(j = 0; j < NUM_CONVERSIONS; j++)
-        {
-        gavl_video_scaler_scale(scaler, frame, frame_1);
-        }
-      timer_stop();
-      }
 #endif
     gavl_video_frame_destroy(frame);
     gavl_video_frame_destroy(frame_1);
